const logger = require('@turnkeyvr/logger');
const MongoDbCouponCodes = new (require('@turnkeyvr/mongodbhelper').MongoDbHelper)('coupon-codes-prod');
const moment = require('moment');

const isProd = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');
const daysToKeep = 5;

const handler = async () => {
    // only runs in production
    if (!isProd) {
        logger.verbose('coupon code sync cleanup only runs in production');
        return;
    }

    logger.verbose('COUPON CODE SYNC CLEANUP START');

    let day = moment().add(-1 * daysToKeep, 'days').format('MM-DD-YYYY');
    let syncVersion = 1;
    let syncOperationsCollectionName;
    let escapiaCollectionName;
    let sfdcCollectionName;
    let mongoConn;
    
    try {
        mongoConn = await MongoDbCouponCodes.connect();
        let dbo = mongoConn.db();
        let existingCollectionNames = (await dbo.collections()).map(c => c.collectionName);

        // delete collections as needed
        while (true) {
            if (!existingCollectionNames.includes(`coupon-code-sync-operations-${day}-v${syncVersion}`)) {
                break;
            }

            syncOperationsCollectionName = `coupon-code-sync-operations-${day}-v${syncVersion}`;
            await dbo.collection(syncOperationsCollectionName).drop();

            escapiaCollectionName = `coupon-code-sync-escapia-${day}-v${syncVersion}`;
            await dbo.collection(escapiaCollectionName).drop();

            sfdcCollectionName = `coupon-code-sync-sfdc-${day}-v${syncVersion}`;
            await dbo.collection(sfdcCollectionName).drop();

            syncVersion += 1;
        }

        mongoConn.close();
    }
    catch (e) {
        try {
            mongoConn.close();
        }
        finally {
            logger.alert('Coupon Code Sync Cleanup Failed');
            logger.alert(e.message);
            return;
        }
    }

    logger.verbose('COUPON CODE SYNC CLEANUP COMPLETE');
};

exports.handler = handler;