const { sleep, resultsEnum } = require('../../util/misc');
const logger = require('@turnkeyvr/logger');
const eachSeries = require('async/eachSeries');

const reportURL = 'https://reports.escapia.com/ReportServer/Pages/ReportViewer.aspx?%2fEscapiaReports%2fAccounting%2fSetup%2fCharge+Template+Unit+Scope&rs%3aCommand=Render';

const downloadChargeTemplateUnitScopeReport = async (browser, downloadPath) => {

    try {
        let page = await browser.newPage();

        await page.goto(reportURL, {waitUntil: 'networkidle0'});


        // wait until report loads
        // sometimes the first click doesn't work (escapia wonkiness) -- if timeout, click again
        try {
            await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 30000});
        }
        catch (e) {
            if (e.message.toLowerCase().indexOf('timeout 30000ms exceeded') === -1) {
                return resultsEnum.FAILURE;
            }
            await page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
                elem.click();
            });
            try {
                await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 180000});
            }
            catch (e2) {
                return resultsEnum.FAILURE;
            }
        }

        logger.debug('inline report loaded');

        // download
        let exportURLBase = await page.evaluate(() => {
            // escapia js code used to find the export url
            return $find('ReportViewerControl')._tryGetInternalViewer().ExportUrlBase;
        });
        if (!exportURLBase || exportURLBase.indexOf('/ReportServer/Reserved.ReportViewerWebControl.axd') === -1) {
            return resultsEnum.FAILURE;
        }

        let downloadPage = await browser.newPage();
        await downloadPage._client.send('Page.setDownloadBehavior', {
            behavior: 'allow',
            downloadPath: downloadPath
        });

        let downloadURL = `https://reports.escapia.com${exportURLBase}CSV`;

        // as bizarre as it is, we're actually expecting a network error here when Escapia closes the connection...
        try {
            await downloadPage.goto(downloadURL, {timeout: 0});
        }
        catch (e) {
            console.log(e.message);
            if (e.message.indexOf('net::ERR_ABORTED') === -1) {
                return resultsEnum.FAILURE;
            }
        }
    }
    catch (e) {
        logger.error(e.message);
        return resultsEnum.FAILURE;
    }

    return resultsEnum.SUCCESS;
};

exports.downloadChargeTemplateUnitScopeReport = downloadChargeTemplateUnitScopeReport;