const { sleep, resultsEnum } = require('../../util/misc');
const logger = require('@turnkeyvr/logger');
const eachSeries = require('async/eachSeries');

const folioBalanceReportURL = 'https://reports.escapia.com/ReportServer/Pages/ReportViewer.aspx?%2fEscapiaReports%2fAccounting%2fReservations%2fFolio+Balance+Report&rs%3aCommand=Render';

const NIGHTCONFIG = {
    Arrival: '1',
    LastNight: '2'
}

// todo: document reportParams obj
// {"reportType":"Folio Balance","reportParams":{"startDate": "6/6/2019","endDate": "7/16/2019", "nightConfig": "Arrival"}}
const downloadFolioBalanceReport = async (browser, downloadPath, reportParams) => {
    if ( typeof reportParams.nightConfig == 'undefined' )
        reportParams.nightConfig = 'LastNight'

    let fileFormat = reportParams.fileFormat ? reportParams.fileFormat : 'CSV'; // EXCELOPENXML, CSV , etc

    try {
        let page = await browser.newPage();

        await page.goto(folioBalanceReportURL, {waitUntil: 'networkidle0'});

        // if reportParams.startDate and reportParams.endDate are populated, fill out on page
        if (reportParams.startDate) {
            await eachSeries(new Array(20), async () => {
                await page.focus('#ReportViewerControl_ctl04_ctl03_txtValue');
                await page.keyboard.press('Delete');
            });

            await page.focus('#ReportViewerControl_ctl04_ctl03_txtValue');
            await page.keyboard.type(reportParams.startDate);
            await page.keyboard.press('Enter');
        }

        await sleep(5000); // todo: required for now, need to check for element being visible

        if (reportParams.endDate) {
            await eachSeries(new Array(20), async () => {
                await page.focus('#ReportViewerControl_ctl04_ctl05_txtValue');
                await page.keyboard.press('Delete');
            });

            await page.focus('#ReportViewerControl_ctl04_ctl05_txtValue');
            await page.keyboard.type(reportParams.endDate);
            await page.keyboard.press('Enter');
        }

        await sleep(5000);  // todo: required for now, need to check for element being visible

        // check "show details"
        let showDetailsChecked = await page.$eval('#ReportViewerControl_ctl04_ctl07_rbTrue', (checkbox) => {
            if (!checkbox) {
                return false;
            }
            checkbox.checked = true;
            return true;
        });
        if (!showDetailsChecked) {
            return resultsEnum.FAILURE;
        }
        
        // check "include cancelled bookings"
        let includeCancelledBookingsChecked = await page.$eval('#ReportViewerControl_ctl04_ctl11_rbTrue', (checkbox) => {
            if (!checkbox) {
                return false;
            }
            checkbox.checked = true;
            return true;
        });
        if (!includeCancelledBookingsChecked) {
            return resultsEnum.FAILURE;
        }

        // select "all offices" for office
        await page.select('#ReportViewerControl_ctl04_ctl15_ddValue', '0');

        // select "last night"
        await page.select('#ReportViewerControl_ctl04_ctl13_ddValue', NIGHTCONFIG[reportParams.nightConfig]);

        // check "show zero balanced folios"
        let showZeroBalancedFoliosChecked = await page.$eval('#ReportViewerControl_ctl04_ctl09_rbTrue', (checkbox) => {
            if (!checkbox) {
                return false;
            }
            checkbox.checked = true;
            return true;
        });
        if (!showZeroBalancedFoliosChecked) {
            return resultsEnum.FAILURE;
        }

        // click on "view report"
        //await page.click('#ReportViewerControl_ctl04_ctl00', {waitUntil: 'networkidle0'});
        await page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
            elem.click();
        });

        // wait until report loads
        // sometimes the first click doesn't work (escapia wonkiness) -- if timeout, click again
        try {
            await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 30000});
        }
        catch (e) {
            if (e.message.toLowerCase().indexOf('timeout 30000ms exceeded') === -1) {
                return resultsEnum.FAILURE;
            }
            await page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
                elem.click();
            });
            try {
                await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 180000});
            }
            catch (e2) {
                return resultsEnum.FAILURE;
            }
        }

        logger.debug('inline report loaded');

        // download
        let exportURLBase = await page.evaluate(() => {
            // escapia js code used to find the export url
            return $find('ReportViewerControl')._tryGetInternalViewer().ExportUrlBase;
        });
        if (!exportURLBase || exportURLBase.indexOf('/ReportServer/Reserved.ReportViewerWebControl.axd') === -1) {
            return resultsEnum.FAILURE;
        }

        let downloadPage = await browser.newPage();
        await downloadPage._client.send('Page.setDownloadBehavior', {
            behavior: 'allow',
            downloadPath: downloadPath
        });

        let downloadURL = 'https://reports.escapia.com' + exportURLBase + fileFormat;
        
        // as bizarre as it is, we're actually expecting a network error here when Escapia closes the connection...
        try {
            await downloadPage.goto(downloadURL, {timeout: 0});
        }
        catch (e) {
            console.log(e.message);
            if (e.message.indexOf('net::ERR_ABORTED') === -1) {
                return resultsEnum.FAILURE;
            }
        }
    }
    catch (e) {
        logger.error(e.message);
        return resultsEnum.FAILURE;
    }

    return resultsEnum.SUCCESS;
};

exports.downloadFolioBalanceReport = downloadFolioBalanceReport;