const { sleep, resultsEnum } = require('../../util/misc');
const logger = require('@turnkeyvr/logger');
const eachSeries = require('async/eachSeries');

const reportURL = 'https://reports.escapia.com/ReportServer?%2FEscapiaReports%2FAccounting%2FOwners%2FOwner%20Ledger%20Report&rs%3aCommand=Render';

// todo: document reportParams obj
// {"reportType":"Owner Ledger","reportParams":{"startDate": "6/6/2019","endDate": "7/16/2019"}}
const downloadOwnerLedgerReport = async (browser, downloadPath, reportParams) => {
    if ( typeof reportParams.nightConfig == 'undefined' )
        reportParams.nightConfig = 'LastNight'

    let fileFormat = reportParams.fileFormat ? reportParams.fileFormat : 'CSV'; // EXCELOPENXML, CSV , etc

    try {
        let page = await browser.newPage();

        await page.goto(reportURL, {waitUntil: 'networkidle0'});

        // select all owners
        // open dropdown
        await page.$eval('#ReportViewerControl_ctl04_ctl03_ddDropDownButton', (dropdown) => {
            dropdown.click();
        });
        // select all owners
        await page.$eval('#ReportViewerControl_ctl04_ctl03_divDropDown_ctl00', (checkbox) => {
            checkbox.click();
        });
        // close dropdown
        await page.$eval('#ReportViewerControl_ctl04_ctl03_ddDropDownButton', (dropdown) => {
            dropdown.click();
        });

        // have to wait for loading icon to go away
        await page.waitForSelector('#ReportViewerControl_AsyncWait_Wait', {visible: true});
        while (true) {
            await sleep(10000);
            let isHidden = await page.$eval('#ReportViewerControl_AsyncWait_Wait', loadingIcon => {
                return loadingIcon.style.display && loadingIcon.style.display === 'none';
            });
            if (isHidden) break;
        }

        // if reportParams.startDate and reportParams.endDate are populated, fill out on page
        if (reportParams.startDate) {
            await eachSeries(new Array(20), async () => {
                await page.focus('#ReportViewerControl_ctl04_ctl05_txtValue');
                await page.keyboard.press('Delete');
            });

            await page.focus('#ReportViewerControl_ctl04_ctl05_txtValue');
            await page.keyboard.type(reportParams.startDate);
            await page.keyboard.press('Enter');

            // have to wait for loading icon to go away
            await page.waitForSelector('#ReportViewerControl_AsyncWait_Wait', {visible: true});
            while (true) {
                await sleep(10000);
                let isHidden = await page.$eval('#ReportViewerControl_AsyncWait_Wait', loadingIcon => {
                    return loadingIcon.style.display && loadingIcon.style.display === 'none';
                });
                if (isHidden) break;
            }
        }

        if (reportParams.endDate) {
            await eachSeries(new Array(20), async () => {
                await page.focus('#ReportViewerControl_ctl04_ctl07_txtValue');
                await page.keyboard.press('Delete');
            });

            await page.focus('#ReportViewerControl_ctl04_ctl07_txtValue');
            await page.keyboard.type(reportParams.endDate);
            await page.keyboard.press('Enter');

            // have to wait for loading icon to go away
            await page.waitForSelector('#ReportViewerControl_AsyncWait_Wait', {visible: true});
            while (true) {
                await sleep(10000);
                let isHidden = await page.$eval('#ReportViewerControl_AsyncWait_Wait', loadingIcon => {
                    return loadingIcon.style.display && loadingIcon.style.display === 'none';
                });
                if (isHidden) break;
            }
        }

        // click on Show Accruals
        await page.$eval('#ReportViewerControl_ctl04_ctl09_rbTrue', (checkbox) => {
            checkbox.click();
        });

        await sleep(5000); // probably not needed but better safe than sorry


        // click on "view report"
        //await page.click('#ReportViewerControl_ctl04_ctl00', {waitUntil: 'networkidle0'});
        await page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
            elem.click();
        });

        // wait until report loads
        // sometimes the first click doesn't work (escapia wonkiness) -- if timeout, click again
        try {
            await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 12 * 60 * 1000});
        }
        catch (e) {
            if (e.message.toLowerCase().indexOf('waiting for') === -1) {
                return resultsEnum.FAILURE;
            }
            await page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
                elem.click();
            });
            try {
                await page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {visible: true, timeout: 720000});
            }
            catch (e2) {
                return resultsEnum.FAILURE;
            }
        }

        logger.debug('inline report loaded');

        // download
        let exportURLBase = await page.evaluate(() => {
            // escapia js code used to find the export url
            return $find('ReportViewerControl')._tryGetInternalViewer().ExportUrlBase;
        });
        if (!exportURLBase || exportURLBase.indexOf('/ReportServer/Reserved.ReportViewerWebControl.axd') === -1) {
            return resultsEnum.FAILURE;
        }

        let downloadPage = await browser.newPage();
        await downloadPage._client.send('Page.setDownloadBehavior', {
            behavior: 'allow',
            downloadPath: downloadPath
        });

        let downloadURL = 'https://reports.escapia.com' + exportURLBase + fileFormat;
        
        // as bizarre as it is, we're actually expecting a network error here when Escapia closes the connection...
        try {
            await downloadPage.goto(downloadURL, {timeout: 0});
        }
        catch (e) {
            logger.error(e.message);
            if (e.message.indexOf('net::ERR_ABORTED') === -1) {
                return resultsEnum.FAILURE;
            }
        }
    }
    catch (e) {
        logger.error(e.message);
        return resultsEnum.FAILURE;
    }

    return resultsEnum.SUCCESS;
};

exports.downloadOwnerLedgerReport = downloadOwnerLedgerReport;