const { sleep, resultsEnum } = require('../../util/misc');
const { waitForReportSpinner } = require('../../util/puppeteerUtil');
const logger = require('@turnkeyvr/logger');
const eachSeries = require('async/eachSeries');
const { retry } = require('../../util/retry');

const rateActionGridReportURL = 'https://reports.escapia.com/ReportServer?%2fEscapiaReports%2fUnits%2fRates+and+Seasons%2fRate+Action+Grid&rs%3aCommand=Render';

// todo: document reportParams obj
// {"reportType":"Folio Balance","reportParams":{"startDate": "6/6/2019","endDate": "7/16/2019"}}
exports.downloadRateActionGridReport = async (browser, downloadPath, reportParams) => {
    let fileFormat = reportParams.fileFormat ? reportParams.fileFormat : 'CSV'; // EXCELOPENXML, CSV , etc

    try {
        let page = await browser.newPage();

        await page.goto(rateActionGridReportURL, {waitUntil: 'networkidle0'});

        await page.select('#ReportViewerControl_ctl04_ctl05_ddValue', '4233');

        await waitForReportSpinner(page)

        await sleep(1000)

        await page.evaluate(() => {
            document.querySelector('#ReportViewerControl_ctl04_ctl07_txtValue').click()
            return true
        });

        await sleep(1000)

        await page.waitForSelector('#ReportViewerControl_ctl04_ctl07_divDropDown_ctl02', {visible: true, timeout: 30000});

        await page.click('#ReportViewerControl_ctl04_ctl07_divDropDown_ctl02')

        await page.click('#ReportViewerControl_ctl09')

        await waitForReportSpinner(page)

        // click view report
        try {
            await retry(
                () => Promise.all([
                    page.$eval('.SubmitButtonCell #ReportViewerControl_ctl04_ctl00', elem => {
                        elem.click();
                    }),
                    page.waitForSelector('#VisibleReportContentReportViewerControl_ctl09', {
                        visible: true,
                        timeout: 30000
                    })
                ]), 3
            );
        }
        catch (ex) {
            return resultsEnum.FAILURE;
        }

        logger.debug('inline report loaded');

        // download
        let exportURLBase = await page.evaluate(() => {
            // escapia js code used to find the export url
            return $find('ReportViewerControl')._tryGetInternalViewer().ExportUrlBase;
        });
        if (!exportURLBase || exportURLBase.indexOf('/ReportServer/Reserved.ReportViewerWebControl.axd') === -1) {
            return resultsEnum.FAILURE;
        }

        let downloadPage = await browser.newPage();
        await downloadPage._client.send('Page.setDownloadBehavior', {
            behavior: 'allow',
            downloadPath: downloadPath
        });

        let downloadURL = 'https://reports.escapia.com' + exportURLBase + fileFormat;

        // as bizarre as it is, we're actually expecting a network error here when Escapia closes the connection...
        try {
            await downloadPage.goto(downloadURL, {timeout: 0});
        }
        catch (e) {
            console.log(e.message);
            if (e.message.indexOf('net::ERR_ABORTED') === -1) {
                return resultsEnum.FAILURE;
            }
        }
    }
    catch (e) {
        logger.error(e.message);
        return resultsEnum.FAILURE;
    }

    return resultsEnum.SUCCESS;
};

