const { setAndGetConfigEscapia, authenticateWithEscapia, logoutOfEscapia } = require('../util/escapiaUtil');
const { getBrowser } = require('../util/puppeteerUtil');
const { retry } = require('../util/retry');
const { mapLineItem } = require('../util/lineItemMapper');
const sfgen = require('@turnkeyvr/salesforcegeneral').promise;
const awsSns = require('@turnkeyvr/aws_sns');
const { promisify } = require("util");
const moment = require('moment');
const { sleep } = require('../util/misc');
const { cancelRez } = require('../util/cancelRezUtil');
const { downloadFolioBalanceReport } = require('../downloadReport/reports/folioBalanceReport');
const fs = require('fs-extra');
const sane = require('sane');
const { resultsEnum } = require('../util/misc');
const logger = require('@turnkeyvr/logger');

const promiseSendSns = promisify(awsSns.publishEmail);

let env = 'staging'
if ( process.env.NODE_ENV.toLowerCase() == 'production' )
    env = 'prod'

const mainHandler = async (event, context) => {
    logger.info('manageCancels main running! ' + env)

    let sfCancelCases = await sfgen.query(`Select Id, Subject, Description, ContactId, Type, Owner.Name,
         Reservation__r.Id, Reservation__r.ReservationNumber__c, Reservation__r.EscapiaNETUnitReservationID__c,
         Reservation__r.Rental__r.Name, Reservation__r.Rental__r.Id, Reservation__r.Rental__r.Escapia_Instance_Id__c,
         Reservation__r.RentValue__c,Reservation__r.Tax__c, Reservation__r.ReservationValue__c, Reservation__r.ReservationBalance__c,
         Reservation__r.ArrivalDate__c, Reservation__r.DepartureDate__c, Reservation__r.Status__c
         from Case
         where Subject in ('guest portal initiated cancel', 'Government Initiated Cancel')
         and Reservation__c!=null and Status='New' and
         Type in ('10% future credit', '100% future credit', '100% Guest refund', 'No refund or credit')
         order by CreatedDate`)

        /*
        let sfCancelCases = await sfgen.query(`Select Id, Subject, Description, ContactId,
            Reservation__r.Id, Reservation__r.ReservationNumber__c, Reservation__r.EscapiaNETUnitReservationID__c,
            Reservation__r.Rental__r.Name, Reservation__r.Rental__r.Id, Reservation__r.RentValue__c,
            Reservation__r.Tax__c, Reservation__r.ReservationValue__c, Reservation__r.ReservationBalance__c,
            Reservation__r.ArrivalDate__c, Reservation__r.DepartureDate__c, Reservation__r.Status__c, Type
            from Case where CaseNumber = '06365059'`)
        */

    logger.info('cases?' + sfCancelCases.length)
    if ( sfCancelCases.length > 0 ) {
        for (let idx=0; idx<sfCancelCases.length; idx++) {
            let item = sfCancelCases[idx]
            // logger.info('item?' + JSON.stringify(item))

            if ( item.Subject == 'Government Initiated Cancel' && item.Owner.Name != 'Robot Cases' ) {
                continue
            }
            else if ( item.Reservation__r.Status__c == 'Canceled' ) {
                await sfgen.update("Case", {Id: item.Id, 
                    Status:'In Progress', 
                    Description: 'Robot did not process because reservation was already canceled.  Please verify that folio is balanced!'})
            }
            else if ( item.Description.indexOf('[DONOTERASE]') == -1 ) {
                if ( item.Reservation__r.Tax__c == null )
                    item.Reservation__r.Tax__c = 0
                item.charges = item.Reservation__r.ReservationValue__c - item.Reservation__r.RentValue__c - item.Reservation__r.Tax__c
                item.reservationId = item.Reservation__r.EscapiaNETUnitReservationID__c.substring(item.Reservation__r.EscapiaNETUnitReservationID__c.indexOf('-') + 1)
                item.payments = 0 // set after we pull folio later
                item.ccPayments = 0 // set after we pull folio later
                item.lineItems = [] // set after we pull folio later
                item.channel = 'TK'
                item.airbnbConfirmNumber = null

                logger.info('sending rez to cancel: ' + item.Type)
                // TESTING ONLY!
                // await workerHandler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(item)}}]}, null)
                await promiseSendSns('arn:aws:sns:us-east-2:673853704728:escapia-robotify-' + env  + '-cancelRez', 'CANCELREZ', JSON.stringify(item))
                break
            }
        }
    }
}

exports.mainHandler = mainHandler

const airbnbHandler = async (event, context) => {
    logger.info('loading airbnb cases!')
    let sfCancelCases = await sfgen.query(`Select Id, Subject, Description, ContactId,
        Reservation__r.Id, Reservation__r.ReservationNumber__c, Reservation__r.EscapiaNETUnitReservationID__c,
        Reservation__r.Rental__r.Escapia_Instance_Id__c,
        Reservation__r.Rental__r.Name, Reservation__r.Rental__r.Id, Reservation__r.RentValue__c,
        Reservation__r.Tax__c, Reservation__r.ReservationValue__c, Reservation__r.ReservationBalance__c,
        Reservation__r.ArrivalDate__c, Reservation__r.DepartureDate__c, Reservation__r.Cancellation_Reason__c, Type
        from Case where Status='New' and Subject like 'Cancel Airbnb Reservation%' order by CreatedDate`) // this last line temporary

    let handledConfirm = {}
    for (let idx=0; idx<sfCancelCases.length; idx++) {
        

        let item = sfCancelCases[idx]
        let confirmNumber = item.Subject.substring(26, item.Subject.indexOf(' ', 26))
        // logger.info('airbnb confirmNumber: ' + confirmNumber + ' [' + item.Subject + ']')

        // there may be dups
        item.airbnbConfirmNumber = confirmNumber
        if ( typeof handledConfirm[confirmNumber] != 'undefined' ) {
            logger.info('filtering same query dup! ' + item.Id)
            await sfgen.update("Case", {Id: item.Id, Status:'Closed', Description: '[ROBOT PROCESSED DUP]'})
            continue
        }
        handledConfirm[confirmNumber] = {}

        // some have rez - some don't...  :(
        if ( item.Reservation__r == null ) {
            let rezArr = await sfgen.query(`Select Id, ReservationNumber__c, EscapiaNETUnitReservationID__c,
            Rental__r.Name, Rental__r.Id, RentValue__c, Tax__c, ReservationValue__c, ReservationBalance__c,
            ArrivalDate__c, DepartureDate__c, Cancellation_Reason__c from Reservation__c where
            External_Confirmation_Number__c='${confirmNumber}'`)

            if ( rezArr.length == 1 ) {
                let rez = rezArr[0]
                item.Reservation__r = rezArr[0]
            }
            else {
                logger.error(`Can't find airbnb rez: ${confirmNumber} for ${item.Id}`)
                return
            }
        }

        if ( item.Reservation__r.Cancellation_Reason__c != null ) {
            logger.info('filtering already cancelled dup! ' + item.Id)
            await sfgen.update("Case", {Id: item.Id, Status:'Closed', Description: '[ROBOT PROCESSED DUP]'})
            continue
        }

        if ( item.Reservation__r.Tax__c == null )
            item.Reservation__r.Tax__c = 0
        item.charges = item.Reservation__r.ReservationValue__c - item.Reservation__r.RentValue__c - item.Reservation__r.Tax__c
        item.reservationId = item.Reservation__r.EscapiaNETUnitReservationID__c.substring(item.Reservation__r.EscapiaNETUnitReservationID__c.indexOf('-') + 1)
        item.payments = 0 // set after we pull folio later
        item.ccPayments = 0 // set after we pull folio later
        item.lineItems = [] // set after we pull folio later
        item.channel = 'Airbnb'

        // logger.info('sending airbnb rez to cancel: ' + JSON.stringify(item))
        // TESTING ONLY!
        // await workerHandler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(item)}}]}, null)
        await promiseSendSns('arn:aws:sns:us-east-2:673853704728:escapia-robotify-' + env + '-cancelRez', 'CANCELREZ', JSON.stringify(item))

        if (idx > 7)
            break
    }
}

exports.airbnbHandler = airbnbHandler

const workerHandler = async (event, context) => {
    console.log('here?')
    let snsEvent = event.Records[0].Sns
    let cancelCase = JSON.parse(snsEvent.Message)
    logger.info('workerHandler doing ' + cancelCase.Id + ';' + cancelCase.Type)

    // this will fail...
    let succeeded = false
    let browser = null
    try {

        // logger.info('rateChangeSet: ' + JSON.stringify(rateChangeSet))
        browser = await getBrowser();
        if (!browser) {
            return;
        }

        logger.info('GOING TO LOGIN?')

        // authenticate with escapia
        let escapiaConfig = setAndGetConfigEscapia(cancelCase.Reservation__r.Rental__r.Escapia_Instance_Id__c)
        let page = await authenticateWithEscapia(await browser.newPage());
        if (!page) {
            logger.info('Something bad happened?')
            await browser.close();
            return;
        }

        logger.info('YIPPEE I SURVIVED LOGGING IN!')

        if ( cancelCase.channel == 'Airbnb') {
            succeeded = await cancelRez(escapiaConfig, page, cancelCase)

            if (succeeded) {
                logger.info('YIPPEE I CANCELLED RESERVATION')
            }
            else if ( cancelCase.cancelErr ) {
                logger.error(`Escapia Cancel Error ${cancelCase.cancelErr} [${cancelCase.Reservation__r.Id}]`)
                let newDescription = cancelCase.Description += '\nError canceling in Escapia:  ' + cancelCase.cancelErr
                let updateResult = await sfgen.update("Case", {Id: cancelCase.Id, Status:'In Progress', Description:newDescription})
            }
        }
        else {
            let downloadReportResult = await downloadFolioReport(browser, cancelCase)
            logger.info('reportDownloadResult ' + JSON.stringify(downloadReportResult))

            if (downloadReportResult) {
                succeeded = await cancelRez(escapiaConfig, page, cancelCase)
                if (succeeded) {
                    logger.info('YIPPEE I CANCELLED RESERVATION')
                }
                else if ( cancelCase.cancelErr ) {
                    logger.error(`Escapia Cancel Error ${cancelCase.cancelErr} [${cancelCase.Reservation__r.Id}]`)
                    let newDescription = cancelCase.Description += '\nError canceling in Escapia:  ' + cancelCase.cancelErr
                    let updateResult = await sfgen.update("Case", {Id: cancelCase.Id, Status:'In Progress', Description:newDescription})
                }
            }
        }

        // logout of escapia then exit browser
        await logoutOfEscapia(page);
        await browser.close();
    }
    catch (ex) {
        if ( !succeeded ) {
            logger.error('processing robot cancel [' + ex + ']: ' + cancelCase.Id + ';' + cancelCase.airbnbConfirmNumber)

            if ( cancelCase.channel == 'Airbnb' && ex.toString().indexOf('#btnCancelReservation') > -1 ) {
                logger.info('closing processed case')
                let updateResult = await sfgen.update("Case", {Id: cancelCase.Id, Status:'Closed'})
            }
           
            if ( browser )
                await browser.close();
        }
    }

    if ( cancelCase.channel == 'Airbnb' && succeeded) {
        cancelCase.Description = cancelCase.Description.replace('[PENDING ROBOT HANDLING]', '')
        let updateResult = await sfgen.update("Case", {Id: cancelCase.Id, Status:'Closed', Description: '[ROBOT PROCESSED]\n\n' + cancelCase.Description.trim()})
        // logger.info('update airbnb cancel case result: ' + JSON.stringify(updateResult))

        updateResult = await sfgen.update("Reservation__c", {Id: cancelCase.Reservation__r.Id, Cancellation_Reason__c:'No longer travelling'})
        // logger.info('update reservation with cancel reason: ' + JSON.stringify(updateResult))

        logger.info(`${snsEvent.MessageId}: done with airbnb rez cancel - success [REZID: ${cancelCase.Reservation__r.Id}]`);
    }

    else if ( cancelCase.channel == 'TK' && succeeded ) {
        // success or no - clear from work queue
        // OWNERID = API user 1
        let updateResult = await sfgen.update("Case", {Id: cancelCase.Id, Status:'Closed', Description: '[ROBOT PROCESSED]', OwnerId:'005i0000008nXsuAAE'})
        // logger.info('update cancel case result: ' + JSON.stringify(updateResult))

        let rezUpdate = {Id: cancelCase.Reservation__r.Id, Cancellation_Reason__c:'No longer travelling'}
        if ( cancelCase.Subject == 'Government Initiated Cancel' )
            rezUpdate.Cancellation_Reason__c = 'Government Mandated'
        updateResult = await sfgen.update("Reservation__c", rezUpdate)
        // logger.info('update reservation with cancel reason: ' + JSON.stringify(updateResult))

        if ( cancelCase.Type == '10% future credit' || cancelCase.Type == '100% future credit' ) {
            let insertResult = await createFutureCreditCase(cancelCase)
            // logger.info('insert future credit case: ' + JSON.stringify(insertResult))
        }

        for (let idx=0; idx<cancelCase.lineItems.length; idx++) {
            let sfLineItem = mapLineItem(cancelCase.lineItems[idx], cancelCase.Id, cancelCase.Reservation__r.Id)
            let liInsertResult = await sfgen.insert("Canceled_Line_Item__c", sfLineItem)
            // logger.info('li liInsertResult: ' + JSON.stringify(liInsertResult))
        }

        logger.info(`${snsEvent.MessageId}: done with rez cancel - success [REZID: ${cancelCase.Reservation__r.Id}]`);
    }
    else {
        logger.info(`${snsEvent.MessageId}: done with rez cancel - error [REZID: ${cancelCase.Reservation__r.Id}]`);
    }

    return
}

exports.workerHandler = workerHandler

async function createFutureCreditCase(cancelCase) {
    const recordType = {
        guestSupport: '0120H000001B4gBQAS'
    }

    const owners = {
        geResolution: '00G0H000006RcZ5UAK'
    }

    const caseObj = {
        TK_Cost__c: cancelCase.payments,
        recordTypeId: recordType.guestSupport,
        OwnerId: owners.geResolution,
        Subject: 'Coronavirus Cancellation',
        contactId: cancelCase.ContactId,
        Reservation__c: cancelCase.Reservation__r.Id,
        OpportunityCase__c: cancelCase.Reservation__r.Rental__r.Id,
        Origin: 'Guest Portal',
        Type: 'Future Stay Credit',
        Priority: 'Medium',
        ParentId: cancelCase.Id,
        Status: 'New'
    };

    if ( cancelCase.Subject = 'Government Initiated Cancel' )
        caseObj.Origin = 'Government'

    if ( cancelCase.Type =='10% future credit' )
        caseObj.Case_Reason__c = 'Partial Credit'
    else if (cancelCase.Type == '100% future credit' )
        caseObj.Case_Reason__c = 'Full Credit'

    caseObj.Description = `Original Reservation
        Rent = ${cancelCase.Reservation__r.RentValue__c}
        Charges = ${cancelCase.charges}
        Tax = ${cancelCase.Reservation__r.Tax__c}
        Total = ${cancelCase.Reservation__r.ReservationValue__c}
        Payments = ${cancelCase.payments}
        Balance= ${cancelCase.Reservation__r.ReservationBalance__c}`

    return await sfgen.insert("Case", caseObj)
}

async function downloadFolioReport(browser, cancelCase) {
    let success = false
    // aws lambda only allows writing to /tmp
    let downloadPath = `/tmp/cancelRobot`;

    await fs.ensureDir(downloadPath, {mode: 0o2775})
    await fs.emptyDir(downloadPath)

    let watcher = sane(downloadPath, {glob: ['**/*.csv', '**/*.xls']});
    let mArrivalDate = moment(cancelCase.Reservation__r.ArrivalDate__c).format('M/D/YYYY')
    // logger.info('mArrivalDate' + mArrivalDate)

    await Promise.all([
        new Promise(resolve => watcher.on('add', async filepath => {
            // logger.debug('SANE download complete: ', filepath);

            // give file time to download...  no way to tell when :| that i can tell
            await sleep(10000)
            watcher.close();

            let data = fs.readFileSync(downloadPath + '/' + filepath, 'utf8');
            // var dataBuffer = Buffer.from(data, 'binary');

            const parse = require('csv-parse/lib/sync')
            const records = parse(data, {
                columns: true,
                skip_empty_lines: true
            })

            let lineItems = []
            let payments = 0
            let ccPayments = 0

            for ( let idx=0; idx<records.length; idx++ ) {
                let li = records[idx]

                if ( li.Reservation_Number == cancelCase.Reservation__r.ReservationNumber__c ) {
                    if ( li.DocumentDescription.indexOf('CC:') > -1 ||
                        li.DocumentDescription.indexOf('Check:') > -1 ||
                        li.DocumentDescription.indexOf('Cash:') > -1 ) {
                        payments += parseFloat(li.Amount.replace(',',''))
                    }

                    if ( li.DocumentDescription.indexOf('CC:') > -1 ) {
                        ccPayments += parseFloat(li.Amount.replace(',',''))
                    }

                    lineItems.push(li)
                }
            }

            cancelCase.payments = Math.abs(payments)
            cancelCase.ccPayments = Math.abs(ccPayments)
            cancelCase.lineItems = lineItems
            success = true

            fs.emptyDirSync(downloadPath)
            resolve()
        })),
        new Promise(async(resolve, reject) => {
            let downloadResults = await downloadFolioBalanceReport
            (browser, downloadPath, {startDate:mArrivalDate, endDate:mArrivalDate, nightConfig:'Arrival'})

            if (downloadResults === resultsEnum.FAILURE) {
                logger.error('failed to download report');
                watcher.close();
                reject()
            }
            else
                resolve()
        }),

    ]);

    return success
}
