const { getBrowser } = require('../util/puppeteerUtil');
const { setAndGetConfigEscapia, authenticateWithEscapia, logoutOfEscapia } = require('../util/escapiaUtil');
const { setUnitGroup } = require('../util/escapiaUnitGroupUtil');
const { queryBq } = require('../util/bigQueryUtil');
const fs = require('fs').promises;
const { retry } = require('../util/retry');
const awsSns = require('@turnkeyvr/aws_sns');
const { promisify } = require("util");
const env = require('@turnkeyvr/env');
const sfgen = require('@turnkeyvr/salesforcegeneral').promise;

const logger = require('@turnkeyvr/logger');
const promiseSendSns = promisify(awsSns.publishEmail);

const isProd = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');

const mainHandler = async () => {
    // const sql = await fs.readFile("./manageDiscounts/sqlGroupDiscounts.txt", "utf8");
    // logger.debug(sql)
    let sfRentals = await sfgen.query(`Select Id, Name, Escapia_Instance_Id__c from Opportunity`)
    let rentalInstanceLookup = {}
    let discountGroupsToInstance = {}
    for ( let idx=0; idx<sfRentals.length; idx++ ) {
        if ( sfRentals[idx].Escapia_Instance_Id__c == null )
            sfRentals[idx].Escapia_Instance_Id__c = '1655'

        rentalInstanceLookup[sfRentals[idx].Name] = sfRentals[idx].Escapia_Instance_Id__c
        if ( typeof discountGroupsToInstance[sfRentals[idx].Escapia_Instance_Id__c] == 'undefined' )
            discountGroupsToInstance[sfRentals[idx].Escapia_Instance_Id__c] = {discountGroupToRental:{}}
    }

    let sfRmSettings = await sfgen.query(`Select Unit_Group_SQL__c from Rate_Mgmt_Automation_Setting__c where Name='Settings (Use This Only!)'`)
    let sfRmSetting = sfRmSettings[0]
    let bqResults = await queryBq(sfRmSetting.Unit_Group_SQL__c);

    bqResults.forEach(row => {
        if (row.Unit_Grop != null) {
            let instanceId = rentalInstanceLookup[row.Name]

            if (instanceId && discountGroupsToInstance[instanceId]) {
                if (typeof discountGroupsToInstance[instanceId].discountGroupToRental[row.Unit_Grop] == 'undefined')
                    discountGroupsToInstance[instanceId].discountGroupToRental[row.Unit_Grop] = []
                discountGroupsToInstance[instanceId].discountGroupToRental[row.Unit_Grop].push(row.Name)
            }
        }
    })

    logger.info('need to process ' + Object.keys(discountGroupsToInstance).length + ' discount groups')

    let currentSet = null
    let setCount = 0
    for ( let iPropt in discountGroupsToInstance ) {
        currentSet = {instanceId: iPropt, groups: {}}

        for ( let propt in discountGroupsToInstance[iPropt].discountGroupToRental ) {
            if ( ++setCount == 10 ) {
                await promiseSendSns(env.config.get('snsTopics.manageDiscountWorkerDispatch'), 'DISCOUNTSET', JSON.stringify(currentSet))

                setCount = 0
                currentSet = {instanceId: iPropt, groups: {}}
            }

            currentSet.groups[propt] = discountGroupsToInstance[iPropt].discountGroupToRental[propt]
        }
    }

    if ( setCount > 0 )
        await promiseSendSns(env.config.get('snsTopics.manageDiscountWorkerDispatch'), 'DISCOUNTSET', JSON.stringify(currentSet))

    return
}

exports.mainHandler = mainHandler

const workerHandler = async (event, context) => {
    let snsEvent = event.Records[0].Sns

    logger.debug(snsEvent.MessageId + ': worker handler called with new set to process! Records?' + event.Records.length);
    let currentSet = JSON.parse(snsEvent.Message)
    let discountGroupToRental = currentSet.groups

    let escapiaConfig = setAndGetConfigEscapia(currentSet.instanceId)
    let browser = await getBrowser();
    if (!browser) {
        return;
    }

    // authenticate with escapia
    let page = await authenticateWithEscapia(await browser.newPage());
    if (!page) {
        await browser.close();
        return;
    }

    // logger.info(snsEvent.MessageId + ': looping through discountGroupToRental')
    for ( let propt in discountGroupToRental ) {
        await retry( () => setUnitGroup(page, propt, snsEvent.MessageId, discountGroupToRental[propt]), 5 )
    }

    // logout of escapia then exit browser
    await logoutOfEscapia(page);
    await browser.close();

    logger.info(snsEvent.MessageId + ': done with discount set');
    return
}

exports.workerHandler = workerHandler