const { getBrowser } = require('../util/puppeteerUtil');
const { setAndGetConfigEscapia, authenticateWithEscapia, logoutOfEscapia } = require('../util/escapiaUtil');
const sfgen = require('@turnkeyvr/salesforcegeneral').promise;
const awsSns = require('@turnkeyvr/aws_sns');
const { promisify } = require("util");
const env = require('@turnkeyvr/env');

const logger = require('@turnkeyvr/logger');
const promiseSendSns = promisify(awsSns.publishEmail);

const isProd = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');

const mainHandler = async (event, context) => {
    logger.info('manageOwnerCleanBilling running!')

    let sfOwnerCleans = await sfgen.query(`SELECT Id, Total_Amount_to_Bill_Owner__c, BLB_Number__c, CreatedDate
        from Housekeeping_Job__c where Bill_Clean_to_Owner__c = 'Yes'
        and Payment_Status__c in ('Paid', 'Paid - Adjusted', 'Paid - Adjusted after payment') 
        and BLB_Number__c = null and CreatedDate >= 2020-09-15T00:00:00.000+0000 
        and Total_Amount_to_Bill_Owner__c>0`)

    logger.info('owner cleans to process in Escapia?' + sfOwnerCleans.length)
    if ( sfOwnerCleans.length > 0 ) {
        for (let idx=0; idx<sfOwnerCleans.length; idx++) {
            let item = sfOwnerCleans[idx]
            
            // await promiseSendSns(env.config.get('snsTopics.processOwnerClean'), 'PROCESSOWNERCLEAN', JSON.stringify(item))
            // dev only
            await workerHandler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(item)}}]}, null)          
        }
    }
}

exports.mainHandler = mainHandler

const workerHandler = async (event, context) => {
    let snsEvent = event.Records[0].Sns
    let serviceOrderCase = JSON.parse(snsEvent.Message)
    logger.info('workerHandler doing ' + serviceOrderCase.Id + ';' + serviceOrderCase.Type)

    // this will fail...
    let successServiceOrderNumber = false
    let browser = null
    try {
        let escapiaConfig = setAndGetConfigEscapia(serviceOrderCase.OpportunityCase__r.Escapia_Instance_Id__c)

        browser = await getBrowser();
        if (!browser) {
            return;
        }

        logger.info('GOING TO LOGIN?')
        // authenticate with escapia
        let page = await authenticateWithEscapia(await browser.newPage());
        if (!page) {
            logger.info('Something bad happened?')
            await browser.close();
            return;
        }

        logger.info('YIPPEE I SURVIVED LOGGING IN!')

        // do the deed
        /*
        try {
            successServiceOrderNumber = await createServiceOrder(escapiaConfig, page, serviceOrderCase) 
        }
        catch (ex) {
            if ( ex == '[NO_RENTAL_FOUND]' ) 
                await setCaseError(serviceOrderCase, 'Rental not found in Escapia!')
            else 
                throw ex
        }
        */

        // logout of escapia then exit browser
        await logoutOfEscapia(page);
        await browser.close();
    }
    catch (ex) {
        if ( !successServiceOrderNumber ) {
            logger.error('processing service order [' + ex + ']: ' + serviceOrderCase.Id)
        }
        
        if ( browser ) 
            await browser.close();
    }

    if ( successServiceOrderNumber )  {
        // close case
        // let caseCloseResult = await sfgen.update("Case", {Id: serviceOrderCase.Id, Status:'Closed', Escapia_Service_Number__c:successServiceOrderNumber})
        let caseCloseResult = await doCaseUpdate(serviceOrderCase, {Id: serviceOrderCase.Id, Status:'Closed', Escapia_Service_Number__c:successServiceOrderNumber})
        logger.info('caseCloseResult? ' + JSON.stringify(caseCloseResult))

        let commentObj = {
            CommentBody__c: successServiceOrderNumber,
            Comment_Created_By__c: '005i0000008nXsuAAE',
            Guest_Impacting__c: false,
            Hide_Comment_from_Owner__c: true,
            Is_case_resolution__c: false,
            Parent_Case__c: serviceOrderCase.Id
        }

        let caseCommentInsertResult = await sfgen.insert('Case_Comment_2__c', commentObj)
        logger.info('caseCommentInsertResult? ' + JSON.stringify(caseCommentInsertResult))

        // TODO ADD CaseComment
        logger.info(`${snsEvent.MessageId}: done with service order - success [REZID: ${serviceOrderCase.Id}]`);
    }
    else {
        // re-assign case?
        logger.info(`${snsEvent.MessageId}: done with service order - error [REZID: ${serviceOrderCase.Id}]`);
        // 00Gi0000004b3qFEAQ: 'Smiles - Service Orders'
        await doCaseUpdate(serviceOrderCase, {Id: serviceOrderCase.Id, OwnerId:'00Gi0000004b3qFEAQ'})
    }

    return
}

exports.workerHandler = workerHandler

const doCaseUpdate = async (oldCase, updateObj) => {
    if ( oldCase.Origin.indexOf('Owner') > -1 )
        updateObj.Origin = 'Internal'

    let result = await sfgen.update("Case", updateObj)
    return result
}

const setCaseError = async (errCase, caseCommentForError) => {
    // 00Gi0000001o799EAA - "Homeowner Support" Queue Id
    // let caseReassignResult = await sfgen.update("Case", {Id: errCase.Id, OwnerId:'00Gi0000001o799EAA'})
    let caseReassignResult = await doCaseUpdate(errCase, {Id: errCase.Id, OwnerId:'00Gi0000001o799EAA'})
    logger.info('caseReassignResult? ' + JSON.stringify(caseReassignResult))

    let commentObj = {
        CommentBody__c: caseCommentForError,
        Comment_Created_By__c: '005i0000008nXsuAAE', // API USER
        Guest_Impacting__c: false,
        Hide_Comment_from_Owner__c: true,
        Is_case_resolution__c: false,
        Parent_Case__c: errCase.Id
    }

    let caseCommentInsertResult = await sfgen.insert('Case_Comment_2__c', commentObj)
    logger.info('caseCommentInsertResult? ' + JSON.stringify(caseCommentInsertResult))
}