const { getBrowser } = require('../util/puppeteerUtil');
const { authenticateWithEscapia, setAndGetConfigEscapia } = require('../util/escapiaUtil');
const { sleep } = require('../util/misc');
const { createCsv } = require('../util/createCsv');
const eachSeries = require('async/eachSeries');
const logger = require('@turnkeyvr/logger');

const runNumber = 2; // this needs to be updated in between runs; todo: make this a command line argument

const inputData = require('./runs/' + runNumber + '/input.json');

const handler = async () => {
    let data = inputData;

    // add logging info for results csv
    data.forEach(record => {
        record.results = 'Fail' // default to fail
        record.notes = ''
    });

    // load browser
    let browser = await getBrowser();
    if (!browser) {
        return;
    }

    // authenticate with escapia
    setAndGetConfigEscapia('1655'); // only running against the main instance
    let page = await authenticateWithEscapia(await browser.newPage());
    if (!page) {
        await browser.close();
        return;
    }

    let results = [];
    await eachSeries(data, async (user) => await processUser(page, user, results));

    createCsv(results, './passwordReset/runs/' + runNumber + '/' + runNumber + '.csv');

    await browser.close();

    logger.debug('results: ', results);

    logger.debug('done');
};

const processUser = async (page, user, results) => {
    logger.debug('user: ', user);

    try {
        // open contact search page
        await page.goto('https://turnkeyvr.escapia.com/WebAgency/FindContacts/Find?Role=Reservationist', {waitUntil: 'networkidle0'});

        // input email
        await page.focus('#txtEmail');
        await page.keyboard.type(user.email);

        // click search button
        await page.click('#butSearch', {waitUntil: 'networkidle0'});

        await sleep(5000); // terrible, bullshit hack; need to replace this

        // click row
        let rowCount = await page.evaluate(() => {
            let elements = document.getElementsByClassName('clickable');
            if (!elements.length || elements.length !== 1) {
                return elements.length;
            }
            elements[0].children[1].click();
            return 1;
        });

        // error handling for no results or multiple results found
        if (rowCount !== 1) {
            let errorMessage = rowCount === 0 ? 'No search results found' : 'Too many search results found';
            throw new Error(errorMessage);
        }

        await page.waitForSelector('#ctlContactWizard_ctl01_pnlPassword');

        await page.evaluate(() => {
            let grandparent = document.getElementById('ctlContactWizard_ctl01_pnlPassword');
            grandparent.firstElementChild.firstElementChild.click();
        });

        await page.waitForSelector('#txtPassword');

        // fill in new password
        await page.focus('#txtPassword');
        await page.keyboard.type(user.password);

        await page.focus('#txtConfirmPassword');
        await page.keyboard.type(user.password);

        // click save button
        await page.click('#btnSave', {waitUntil: 'load'});

        user.results = 'Success';
        results.push(user);
    }
    catch (e) {
        logger.error(e);
        user.results = 'Failure';
        user.notes = e;
        results.push(user);
    }
};


exports.handler = handler;