const logger = require('@turnkeyvr/logger');
const sfgen = require('@turnkeyvr/salesforcegeneral').promise;

const isProd = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');

exports.handler = async (event, context) => {
    if (!isProd) {
        logger.debug('advancedDepositReportWorker only runs in production');
        return;
    }
    
    const mongoDbAdvancedDeposits = new (require('@turnkeyvr/mongodbhelper').MongoDbHelper)('advancedDepositsBuilder-prod')

    let snsEvent = event.Records[0].Sns
    let workItem = JSON.parse(snsEvent.Message)
    
    logger.info('running adb report builder: ' + JSON.stringify(workItem))
    
    let sfReservations = await sfgen.query(`Select EscapiaNETUnitReservationID__c,
        CancellationDateTime__c, Canceled_Reservation_Value__c, ReservationNumber__c, 
        ReservationValue__c, ReservationBalance__c, ArrivalDate__c, DepartureDate__c, 
        SourceSubCategory__c, BookDate__c, Rental__r.Name, Rental__r.MarketObject__r.Name, 
        Rental__r.MarketObject__r.Top_Market__r.Name, Rental__r.MarketObject__r.State_Abbr__c 
        from Reservation__c 
        where 
        SourceSubCategory__c != 'Owner' and Is_Hold__c = false and 
        DepartureDate__c >= ${workItem.startDate} and 
        DepartureDate__c <= ${workItem.endDate}`)

    logger.info('loaded sf rez: ' + sfReservations.length)
    let lineItemCollName = `escapiaLineItems_${workItem.runDate}`    
    let rezArr = []
    for ( let idx=0; idx<sfReservations.length; idx++ ) {
        let rez = sfReservations[idx]
        rezArr.push(rez.EscapiaNETUnitReservationID__c)
    }
    
    let escapiaLineItems = await mongoDbAdvancedDeposits.find(lineItemCollName, 
        {'Type__c':{ $in:['payment', 'fee', 'refund']}, 'Name':{ $in:rezArr}})
    logger.info('loaded lineItems for rez: ' + escapiaLineItems.length)

    let liRef = {}
    for ( let eidx=0; eidx<escapiaLineItems.length; eidx++ ) {
        let lineItem = escapiaLineItems[eidx]
        if ( typeof liRef[lineItem.Name] == 'undefined' ) 
            liRef[lineItem.Name] = {CreditAmt: 0, items:[]}
        
        if ( lineItem.Description__c == 'TKVR - Future Stay Credit' )
            liRef[lineItem.Name].CreditAmt += lineItem.Amount__c

        lineItem.PaymentType = null
        if ( lineItem.Type__c == 'payment' ) {
            if ( lineItem.Description__c.indexOf('CC:') > -1 )
                lineItem.PaymentType = 'CC'
            if ( lineItem.Description__c.indexOf('Check:') > -1 )
                lineItem.PaymentType = 'Check'
            if ( lineItem.Description__c.indexOf('Cash:') > -1 ) 
                lineItem.PaymentType = 'Cash'
        }

        liRef[lineItem.Name].items.push(lineItem)
    }

    let adrRows = []
    for ( let idx=0; idx<sfReservations.length; idx++ ) {
        let sfRez = sfReservations[idx]
        
        if ( sfRez.Rental__r.MarketObject__r.Top_Market__r == null)
            sfRez.Rental__r.MarketObject__r.Top_Market__r = {Name: sfRez.Rental__r.MarketObject__r.Name}

        if ( typeof liRef[sfRez.EscapiaNETUnitReservationID__c] != 'undefined' ) {
            for ( let eidx=0; eidx<liRef[sfRez.EscapiaNETUnitReservationID__c].items.length; eidx++ ) {
                let li = liRef[sfRez.EscapiaNETUnitReservationID__c].items[eidx]
                
                adrRows.push({
                    RentalName: sfRez.Rental__r.Name,
                    Type:li.Type__c,
                    Reservation:sfRez.ReservationNumber__c,
                    ReservationStatus:sfRez.Status__c,
                    ReservationValue:sfRez.ReservationValue__c,
                    CanceledReservationValue:sfRez.Canceled_Reservation_Value__c,
                    ReservationBalance:sfRez.ReservationBalance__c,
                    FutureStayCredit: liRef[sfRez.EscapiaNETUnitReservationID__c].CreditAmt,
                    ArrivalDate:sfRez.ArrivalDate__c,
                    DepartureDate:sfRez.DepartureDate__c,
                    SourceSubCategory:sfRez.SourceSubCategory__c,
                    MarketName:sfRez.Rental__r.MarketObject__r.Name,
                    TopMarketName:sfRez.Rental__r.MarketObject__r.Top_Market__r.Name,
                    StateAbbr:sfRez.Rental__r.MarketObject__r.State_Abbr__c,
                    LiDescription: li.Description__c,
                    Amount:li.Amount__c,
                    Posting:li.Posting_Date__c
                })  
            }
        }
    }

    let adrCollName = `advancedDepositReport_${workItem.runDate}`  
    if ( adrRows.length > 0 )
        await mongoDbAdvancedDeposits.insertMany(adrCollName, adrRows) 
    await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'processed'})
    await mongoDbAdvancedDeposits.close()
}

