const env = require('@turnkeyvr/env');
const logger = require('@turnkeyvr/logger');
const { promisify } = require("util");
const awsSns = require('@turnkeyvr/aws_sns');
const papa = require('papaparse');
const util = require('util');

const promiseSendSns = promisify(awsSns.publishEmail);

const isProd = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');

exports.handler = async (event, context) => {
    if (!isProd) {
        logger.debug('advancedDepositReportWriter only runs in production');
        return;
    }

    const mongoDbAdvancedDeposits = new (require('@turnkeyvr/mongodbhelper').MongoDbHelper)('advancedDepositsBuilder-prod')

    let snsEvent = event.Records[0].Sns
    let workItem = JSON.parse(snsEvent.Message)

    let adrCollName = `advancedDepositReport_${workItem.runDate}`  

    let reportRows = await mongoDbAdvancedDeposits.find(adrCollName, {})
    logger.info('running adr report writer for ' + adrCollName + ' rows: ' + reportRows.length)
   
    let fileName = `advancedDepositReportRunDate_${workItem.runDate}_departsAfter_${workItem.departsAfter}.csv`

    logger.info('parse csv')
    var csv = papa.unparse(reportRows)
    logger.info('make buffer')
    let dataBuffer = Buffer.from(csv, 'binary');

    const S3 = require('aws-sdk/clients/s3')
    const s3 = new S3()
    s3.putObject = util.promisify(s3.putObject);

    const response = await s3.putObject({Bucket: 'robotify-escapia-reports', Key: fileName, Body: dataBuffer})
    logger.info('response:' + JSON.stringify(response))

    let subject = 'Advanced Deposit Report!'
    let emailBody = `Hot off the press!\nhttps://robotify-escapia-reports.s3.us-east-2.amazonaws.com/${fileName}`
    await promiseSendSns(env.config.get('snsTopics.sendAdvancedDepositsReport'), subject, emailBody)
    logger.info('sent email')

    await mongoDbAdvancedDeposits.close()
}