
const env = require('@turnkeyvr/env');
const { getBrowser } = require('../util/puppeteerUtil');
const { authenticateWithEscapia, logoutOfEscapia } = require('../util/escapiaUtil');
const { mapLineItem } = require('../util/lineItemMapper');
const sfgen = require('@turnkeyvr/salesforcegeneral').promise;
const awsSns = require('@turnkeyvr/aws_sns');
const { promisify } = require("util");
const moment = require('moment');
const { sleep } = require('../util/misc');
const { downloadFolioBalanceReport } = require('../downloadReport/reports/folioBalanceReport');
const fs = require('fs-extra');
const sane = require('sane');
const { resultsEnum } = require('../util/misc');
const csvtojson=require('csvtojson')

const mongoDbAdvancedDeposits = new (require('@turnkeyvr/mongodbhelper').MongoDbHelper)('advancedDepositsBuilder-prod')

const logger = require('@turnkeyvr/logger');
const promiseSendSns = promisify(awsSns.publishEmail);
const FOLIO_REPORT_BATCH_SIZE = 30 // days - same for running escapia reports And building our advanced deposit report
const ADR_BUILDER_BATCH_SIZE = 90

const mainHandler = async (event, context) => {
    logger.info('processLineItems is temporarily disabled until I get accounting requirements');
    return

    let runDate = moment().format('YYYY-MM-DD')

    let departAfterDate = moment()
    if ( departAfterDate.date() < 15 ) 
        departAfterDate = departAfterDate.date(1).subtract(1, 'month').format('YYYY-MM-DD')
    else 
        departAfterDate = departAfterDate.date(1).format('YYYY-MM-DD')

    /*
    const worker = require('./advancedDepositReportWorker');
    const ObjectId = require('mongodb').ObjectID
    let workItemArr = await mongoDbAdvancedDeposits.find('workUnits', {_id: ObjectId("5eab682fc22467000799eca5")}) 
    await worker.handler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(workItemArr[0])}}]}, null)
    return
    */   
   
    // await mongoDbAdvancedDeposits.deleteManyByFilter(`advancedDepositReport_2020-05-01`, {}) 
    // return

    // const worker = require('./advancedDepositReportWriter');
    // await worker.handler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify({runDate:runDate, departsAfter:departAfterDate})}}]}, null)
    // await promiseSendSns(env.config.get('snsTopics.writeAdvancedDepositReport'), 'ADR_WRITER', JSON.stringify({runDate:runDate, departsAfter:departAfterDate}))
    // return

    let runDateResult = await mongoDbAdvancedDeposits.find('processRunDates', {runDate:runDate})

    let currentRunObj = null
    // already ran current runData
    // if (false) {
    if ( runDateResult.length > 0 ) {
        currentRunObj = runDateResult[0]

        let hasMoreWork = await spawnWorkersIfWorkToDo(runDate, 1) 
        if ( !hasMoreWork ) {
            let runDateResult = await mongoDbAdvancedDeposits.find('processRunDates', {runDate:runDate})
            
            if ( runDateResult[0].status == 'building' ) {
                let workUnits = await mongoDbAdvancedDeposits.find('workUnits', {runDate:runDate, status:'queued'})

                if ( workUnits.length == 0 ) {
                    let updateResult = await mongoDbAdvancedDeposits.updateOneById('processRunDates', runDateResult[0]._id, {status:'complete', endProcessTime:moment().format()})
                    await promiseSendSns(env.config.get('snsTopics.writeAdvancedDepositReport'), 'ADR_WRITER', JSON.stringify({runDate:runDate, departsAfter:departAfterDate}))
                }
                else {
                    logger.info('no more work to do but my buddies still working?  I hope.')
                }
            }
        }
    }
    // create a new day's work
    else {
        // TODO - alert via papertrail if yesterday's work was not complete!

        // purge rolling - only need last 2 days
        let purgeDate = moment().subtract(3, 'days').format('YYYY-MM-DD')
        let purgeCollName = `escapiaLineItems_${purgeDate}`
        if ( await mongoDbAdvancedDeposits.collectionExists(purgeCollName) )
            await mongoDbAdvancedDeposits.dropCollection(purgeCollName)
        
        purgeCollName = `advancedDepositReport_${purgeDate}`
        if ( await mongoDbAdvancedDeposits.collectionExists(purgeCollName) )
            await mongoDbAdvancedDeposits.dropCollection(purgeCollName)
    
        let adrCollName = `advancedDepositReport_${runDate}` 
        if ( await mongoDbAdvancedDeposits.collectionExists(adrCollName) )
            await mongoDbAdvancedDeposits.dropCollection(adrCollName)   
        await mongoDbAdvancedDeposits.createCollection(adrCollName)

        let lineItemCollName = `escapiaLineItems_${runDate}`
        // this check/delete is only needed during dev or re-processing      
        if ( await mongoDbAdvancedDeposits.collectionExists(lineItemCollName) )
            await mongoDbAdvancedDeposits.dropCollection(lineItemCollName)
        await mongoDbAdvancedDeposits.createCollection(lineItemCollName)
        await mongoDbAdvancedDeposits.createIndex(lineItemCollName, {Type__c:1})
        await mongoDbAdvancedDeposits.createIndex(lineItemCollName, {Name:1})
        await mongoDbAdvancedDeposits.createIndex(lineItemCollName, {EscapiaLiId:1}, {background:true, unique:true})
        await mongoDbAdvancedDeposits.deleteManyByFilter('workUnits', {})

        currentRunObj = {runDate: runDate, status:'building', startProcessTime: moment().format()}
        await mongoDbAdvancedDeposits.insertMany('processRunDates', [currentRunObj])
    
        // format('YYYY-MM-DD')
        let sfReservations = await sfgen.query(`Select DepartureDate__c
            from Reservation__c 
            where SourceSubCategory__c != 'Owner' and Is_Hold__c = false
            and DepartureDate__c != null
            order by DepartureDate__c desc limit 1`)

        let lastDepartureDate = sfReservations[0].DepartureDate__c
        logger.info('lastDepartureDate ' + lastDepartureDate)

        await buildFolioReportWorkItems(lastDepartureDate, runDate)
        await buildADRWorkItems(lastDepartureDate, runDate, departAfterDate) 
        
        await spawnWorkersIfWorkToDo(runDate, 1) 
        logger.info('done making work units!')
    }
}
exports.mainHandler = mainHandler

async function buildFolioReportWorkItems(lastDepartureDate, runDate) {
    let startDate = moment().subtract(1, 'month').date(1)

    sfReservations = await sfgen.query(`Select ArrivalDate__c
        from Reservation__c 
        where SourceSubCategory__c != 'Owner' and Is_Hold__c = false
        and DepartureDate__c != null and DepartureDate__c > ${startDate.format('YYYY-MM-DD')}
        order by ArrivalDate__c limit 1`)

    startDate = moment(sfReservations[0].ArrivalDate__c)
    let endDate = moment(startDate).add(FOLIO_REPORT_BATCH_SIZE, 'days')

    logger.info('new startDate ' + startDate)

    let workItemArr = []
    while ( true ) {
        logger.info('FOLIO:' + startDate.format() + ' -- ' + endDate.format())
        // create work item        
        workItemArr.push({runDate: runDate, 
            type:'folioRunner',
            status:'unprocessed',
            startDate: startDate.format('YYYY-MM-DD'), 
            endDate: endDate.format('YYYY-MM-DD')})

        startDate = moment(endDate).add('1', 'day')
        endDate.add(FOLIO_REPORT_BATCH_SIZE, 'days')
    
        if ( startDate.isSameOrAfter(lastDepartureDate) )
            break
    }

    let insertResult = await mongoDbAdvancedDeposits.insertMany('workUnits', workItemArr)
    return
}

async function buildADRWorkItems(lastDepartureDate, runDate, departAfterDate) {
    let startDate = moment(departAfterDate)
    let endDate = moment(startDate).add(ADR_BUILDER_BATCH_SIZE, 'days')

    let workItemArr = []
    while ( true ) {
        logger.info('ADRW:' + startDate.format() + ' -- ' + endDate.format())
        // create work item        

        workItemArr.push({runDate: runDate, 
            type:'advancedDepositBuilder',
            status:'unprocessed',
            startDate: startDate.format('YYYY-MM-DD'), 
            endDate: endDate.format('YYYY-MM-DD')})

        if ( startDate.isSameOrAfter(lastDepartureDate) )
            break

        startDate = moment(endDate).add('1', 'day')
        endDate.add(ADR_BUILDER_BATCH_SIZE, 'days')
    }

    let insertResult = await mongoDbAdvancedDeposits.insertMany('workUnits', workItemArr)
    return
}

async function spawnWorkersIfWorkToDo(runDate, numWorkers) {
    // for now - any "queued" item is "stuck" and needs to be attemped again 
    let workUnits = await mongoDbAdvancedDeposits.find('workUnits', {runDate:runDate, type:'folioRunner', status:{$in: ['unprocessed']}}, {limit:1})
    
    let foundWork = false

    if ( workUnits.length == 1 ) {
        let workItem = workUnits[0]
               
        if ( !hasReservationsForDates(workItem) ) {
            logger.info(`no rez between ${workItem.startDate} and ${workItem.endDate}`)
            let updateResult = await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'processed'})
        }
        else {
            foundWork = true

            // yes - some small chance of a race condition if more than one process running (serial for now)
            await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'queued'})

            // await workerHandler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(workUnits[idx])}}]}, null)
            await promiseSendSns(env.config.get('snsTopics.runFolioWorker'), 'FOLIOWORKER', JSON.stringify(workItem))
        }
    }
    else {
        workUnits = await mongoDbAdvancedDeposits.find('workUnits', {runDate:runDate, type:'advancedDepositBuilder', status:{$in: ['unprocessed']}}, {limit:1})
        
        if ( workUnits.length == 1 ) {
            let workItem = workUnits[0]
            if ( !hasReservationsForDates(workItem) ) {
                logger.info(`no rez between ${workItem.startDate} and ${workItem.endDate}`)
                let updateResult = await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'processed'})
            }
            else {
                foundWork = true

                // yes - some small chance of a race condition if more than one process running (serial for now)
                await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'queued'})

                // await workerHandler({Records: [{Sns: {MessageId: '1234', Message: JSON.stringify(workUnits[idx])}}]}, null)
                await promiseSendSns(env.config.get('snsTopics.buildAdvanceDepositsReport'), 'AdvanceDepositsReportWorker', JSON.stringify(workItem))
            }
        }
    }

    async function hasReservationsForDates(workItem) {
        let sfReservations = await sfgen.query(`Select ID from Reservation__c where 
            SourceSubCategory__c != 'Owner' and Is_Hold__c = false and 
            ArrivalDate__c >= ${workItem.startDate} and ArrivalDate__c <= ${workItem.endDate} limit 1`)
            
        return (sfReservations.length == 1)  
    }

    return foundWork
}

const workerHandler = async (event, context) => {
    let snsEvent = event.Records[0].Sns
    let workItem = JSON.parse(snsEvent.Message)
    logger.info('lineitem processor worker running: ' + JSON.stringify(workItem))

    // this will fail...
    let succeeded = false
    let browser = null
    try {
        // logger.info('rateChangeSet: ' + JSON.stringify(rateChangeSet))
        browser = await getBrowser();
        if (!browser) {
            return;
        }

        logger.info('GOING TO LOGIN?')

        // authenticate with escapia
        let page = await authenticateWithEscapia(await browser.newPage());
        if (!page) {
            logger.info('Something bad happened?')
            await browser.close();
            return;
        }

        logger.info('YIPPEE I SURVIVED LOGGING IN!')

        // ex: {"res-1234": {items:[]}
        let lineItemsPerRez = await downloadFolioReport(browser, moment(workItem.startDate), moment(workItem.endDate))
        // logger.info('reportDownloadResult ' + JSON.stringify(downloadReportResult))

        // logout of escapia then exit browser
        await logoutOfEscapia(page);
        await browser.close();
        browser = null

        console.log( 'rez to process?' + Object.keys(lineItemsPerRez).length )
        if ( Object.keys(lineItemsPerRez).length > 0 ) {
            let resultArr = []
            for ( let propt in lineItemsPerRez ) {
                for (let idx=0; idx<lineItemsPerRez[propt].items.length; idx++ ) {
                    resultArr.push(handleLineItem(lineItemsPerRez[propt].items[idx]))
                }
            }

            await mongoDbAdvancedDeposits.insertMany(`escapiaLineItems_${workItem.runDate}`, resultArr)
        }
        else {
            // warning:  escapia doesn't return rez in this report that don't have lineitems
            logger.info(`folio report came back empty?! ${workItem.startDate} - ${workItem.endDate} (might be okay)`)
        }

        let updateResult = await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'processed'})
        logger.info('finished workUnit ' + workItem.startDate + ' :: ' + workItem.endDate)
    }
    catch (ex) {
        if ( !succeeded ) {
            logger.error('processing folio report [' + ex + ']')

            if ( browser )
                await browser.close();

            let updateResult = await mongoDbAdvancedDeposits.updateOneById('workUnits', workItem._id, {status:'unprocessed'})
        }
    }

    return
}
exports.workerHandler = workerHandler

async function downloadFolioReport(browser, startDate, endDate) {
    let success = false
    // aws lambda only allows writing to /tmp
    let downloadPath = `/tmp/advancedDeposits_` + 
        startDate.format('YYYY_MM_DD') + '_to_' +
        endDate.format('YYYY_MM_DD')

    await fs.ensureDir(downloadPath, {mode: 0o2775})
    await fs.emptyDir(downloadPath)

    let watcher = sane(downloadPath, {glob: ['**/*.csv', '**/*.xls']});
    let mArrivalDate = startDate.format('M/D/YYYY')
    let mDepartureDate = endDate.format('M/D/YYYY')

    await Promise.all([
        new Promise(resolve => watcher.on('add', async filepath => {
            // logger.debug('SANE download complete: ', filepath);

            // give file time to download...  no way to tell when :| that i can tell
            await sleep(10000)
            watcher.close();

            const records = await csvtojson().fromFile(downloadPath + '/' + filepath);
            
            let rezColObj = {}
            for ( let idx=0; idx<records.length; idx++ ) {
                let li = records[idx]
                // console.log(li.Reservation_Number)
                if ( typeof rezColObj[li.Reservation_Number] == 'undefined' ) 
                    rezColObj[li.Reservation_Number] = {items:[]}
                
                // logger.info('JSON.stringify(li):' + JSON.stringify(li))
                // console.log('li.Reservation_ID:' + li["Reservation_ID"])
                rezColObj[li.Reservation_Number].items.push(li)
            }

            // console.log('Object.keys(rezColObj).length: ' + Object.keys(rezColObj).length)
            success = rezColObj

            fs.emptyDirSync(downloadPath)
            resolve()
        })),
        new Promise(async(resolve, reject) => {
            let downloadResults = await downloadFolioBalanceReport
                (browser, downloadPath, {startDate:mArrivalDate, endDate:mDepartureDate, nightConfig:'Arrival'})

            if (downloadResults === resultsEnum.FAILURE) {
                logger.error('failed to download report');
                watcher.close();
                reject()
            }
            else
                resolve()
        }),

    ]);

    return success
}

function handleLineItem(lineItem) {
    // logger.info('handleLineItem:' + lineItem.Reservation_ID)
    
    var sfObj = {};
    var rezId = '1655-' + lineItem.Reservation_ID;
    // this.orderCount[rezId] = (this.orderCount[rezId]) ? ++this.orderCount[rezId] : 1;
    // sfObj.Escapia_Order__c = this.orderCount[rezId];
    sfObj.Name = rezId;//used to lookup Rez ID with trigger, updated to Type__c in SF
    sfObj.Description__c = formatDescription(lineItem.DocumentDescription);
    sfObj.Posting_Date__c = formatDate(lineItem.PostingDate);
    sfObj.Start_Date__c= formatDate(lineItem.Start_Date);
    sfObj.End_Date__c = formatDate(lineItem.End_Date);
    sfObj.Escapia_Creation_Date__c = formatDate(lineItem.Creation_Date);
    sfObj.Amount__c = formatCurrency(lineItem.Amount);
    sfObj.EscapiaLiId = formatDescription(lineItem.Document_ID)

    sfObj.Type__c = (sfObj.Amount__c === null) ? 'Unknown' : getType(sfObj,lineItem.Booking_Number);
    return sfObj;
}

var dateFormat = {
    escapia: 'M/D/YYYY',
    sf: 'YYYY-MM-DD'
};

function formatCurrency(value){
    if (isEmpty(value)) return null;
    var parsed = parseFloat(value.replace(/,/g,''));
    return (isNaN(parsed)) ? null : parsed;
}

function formatDate(value){
    return (value && value.length>0) ? moment(value,dateFormat.escapia).format(dateFormat.sf) : null;
}

function isEmpty(value){
    return (value === undefined || value === null || value.length === 0);
}

function formatDescription(value){
    if (isEmpty(value)) return null;
    var updated = value.replace(/,/g,' ');
    return updated.replace(/["']/,'').trim();
}

function getBookingType(description,amount){
    var checks = [
        {search: /vendor fee and tax/i,type:'fee'},
        {search: /tax/i,type:'tax'},
        {search: /county/i,type:'tax'},
        {search: /city/i,type:'tax'},
        {search: /TX - PortA - Sunflower Beach/,type:'tax'},
        {search: /fee/i,type:'fee'},
        {search: /clean/i,type:'fee'},
        {search: /deposit/i,type:'fee'},
        {search: /vendor/i,type:'fee'},
        {search: /tablet/i,type:'fee'},
        {search: /golf/i,type:'fee'},
        {search: /keep/i,type:'fee'},
        {search: /TAC - Airbnb/,type:'fee'},
        {search: /keep/i,type:'fee'},
        {search: [/from/i,/to/im],type:'rent'}
    ];
    var type = findType(description,checks);
    if (!isEmpty(type)) return type;
    return (amount > 0) ? 'fee' : 'refund';
}

function getRezType(description,amount){
    if (amount < 0) return 'payment';
    var checks = [
        {search: /refund/i,type:'refund'},
        {search: /fee/i,type:'fee'},
        {search: /charge/i,type:'fee'},
        {search: /clean/i,type:'fee'}
    ];
    var type = findType(description,checks);
    return (!isEmpty(type)) ? type : 'refund';
}

function getType(obj,bookingNumber){
    var description = obj.Description__c;
    if (isEmpty(description)) return null;
    var amount = obj.Amount__c;
    return (!isEmpty(bookingNumber)) ? getBookingType(description,amount) : getRezType(description,amount);
}

function findType(str,arrSearch){
    var found = arrSearch.find(function(obj){
        if (!(obj.search instanceof Array)) obj.search = [obj.search];
        var matches = obj.search.filter(function(search){
                return (str.match(search) !== null);
        });
        return (matches.length === obj.search.length);
    });
    return (!isEmpty(found)) ? found.type : null;
}

