const { getBrowser } = require('../util/puppeteerUtil');
const { authenticateWithEscapia } = require('../util/escapiaUtil');
const { sleep } = require('../util/misc');
const { createCsv } = require('../util/createCsv');
const eachSeries = require('async/eachSeries');
const logger = require('@turnkeyvr/logger');
const { getSelectOptions } = require('../util/selectOptionsUtil');

const runNumber = 2; // this needs to be updated in between runs; todo: make this a command line argument

const inputData = require('./run/input.json');

const handler = async () => {
    let data = inputData;

    // add logging info for results csv
    data.forEach(record => {
        record.results = 'Fail' // default to fail
        record.notes = ''
    });

    // load browser
    let browser = await getBrowser();
    if (!browser) {
        return;
    }

    // authenticate with escapia
    let page = await authenticateWithEscapia(await browser.newPage());
    if (!page) {
        await browser.close();
        return;
    }

    let results = [];
    await eachSeries(data, async (user) => await processUser(page, user, results));

    createCsv(results, './passwordReset/runs/' + runNumber + '/' + runNumber + '.csv');

    await browser.close();

    logger.debug('results: ', results);

    logger.debug('done');
};

const processUser = async (page, user, results) => {
    logger.info('user: ', user.email);

    try {
        // open contact search page
        await page.goto('https://turnkeyvr.escapia.com/WebAgency/FindContacts/Find?Role=Reservationist', {waitUntil: 'networkidle0'});

        await page.waitForSelector('#lstOffices');
        await page.select('#lstOffices', '0')

        await page.waitForSelector('#lstRoleSelector');
        await page.select('#lstRoleSelector', '0')

        // input email
        await page.focus('#txtEmail');
        await page.keyboard.type(user.email);

        // click search button
        await page.click('#butSearch', {waitUntil: 'networkidle0'});

        await sleep(5000); // terrible, bullshit hack; need to replace this

        // click row
        let rowCount = await page.evaluate(() => {
            let elements = document.getElementsByClassName('clickable');
            return elements.length;
        });

        // error handling for no results or multiple results found
        if (rowCount == 0) {
            let errorMessage = rowCount === 0 ? 'No search results found' : 'Too many search results found';
            return
        }

        await page.evaluate(() => {
            let elements = document.getElementsByClassName('clickable');
            elements[0].children[1].click();
            return
        })

        logger.info('wait for roles')
        await page.waitForSelector('#ctlContactWizard_ctl02_pnlRoles > li > a');

        await page.click('#ctlContactWizard_ctl02_pnlRoles > li > a')

        logger.info('dialog?')
        await Promise.all([
            new Promise(resolve => page.once('dialog', function(dialog, resolve) {
                dialog.accept()
                resolve()
            }, resolve)),
            page.click('#ctlContactWizard_ctl02_pnlRoles > li > a')
        ]);

        await page.waitForSelector('select#lstDualList_RightBox > option');
        let selectArr = []
        let options = await getSelectOptions(page, 'select#lstDualList_RightBox > option' );
        for ( let idx=0; idx<options.namesAndValues.length; idx++ ) {
            selectArr.push(options.namesAndValues[idx].value)
        }

        logger.info('selectArr: ' + JSON.stringify(selectArr))
        await page.select('#lstDualList_RightBox', ...selectArr);
        await page.click('#lstDualList_MoveLeft')
        await page.click('#Button1', {waitUntil: 'load'})

        await sleep(5000)

        logger.info('wait for save button')
        await page.waitForSelector('#SaveBtn')
        logger.info('click for save button')
        await page.click('#SaveBtn', {waitUntil: 'load'});

        user.results = 'Success';
        results.push(user);

        // error handling for no results or multiple results found
        if ( rowCount > 1 ) {
            logger.info('duplicate run for ' + user.email)
            await processUser(page, user, results);
        }

        return
    }
    catch (e) {
        logger.error(e);
        user.results = 'Failure';
        user.notes = e;
        results.push(user);
    }
};


exports.handler = handler;