const {BigQuery} = require('@google-cloud/bigquery');
const logger = require('@turnkeyvr/logger');
const env = require('@turnkeyvr/env');
const { config } = require('@turnkeyvr/env');

let _bqClient = null

exports.queryBq = async (sql) => {
    let bqClient = await getBqClient()

    // For all options, see https://cloud.google.com/bigquery/docs/reference/rest/v2/jobs/query
    const options = {
        query: sql
    };

    // Run the query as a job
    const [job] = await bqClient.createQueryJob(options);
    // logger.debug(`Job ${job.id} started.`);

    // Wait for the query to finish
    const [rows] = await job.getQueryResults();

    // Print the results
    // logger.debug('Rows:?');
    // rows.forEach(row => logger.debug(JSON.stringify(row)));

    return rows
}

async function getBqClient() {
    function nSlashToNewLine(str) {
        return str.replace(/\\n/g, "\n");
    }

    if ( _bqClient != null )
        return _bqClient
    // _bqClient = new BigQuery();

    _bqClient = new BigQuery({
        projectId: "turnkey-data-warehouse",
        credentials: {
            private_key: nSlashToNewLine(config.get("google.bigQuery.credentials.private_key")),
            client_email: config.get("google.bigQuery.credentials.client_email"),
            projectId: "turnkey-data-warehouse"
        }
    });

    return _bqClient
}