const logger = require('@turnkeyvr/logger');
const moment = require('moment');
const { sleep } = require('../util/misc');
const { retry } = require('../util/retry');
const { getSelectOptions } = require('../util/selectOptionsUtil');

let dynamicPageSettings = {
    // TK
    '1655': { cancellationFee:'39847',
        futureCredit:'96217' },
    // MARG
    '2685': { cancellationFee:'100195',
        futureCredit:'100196' },
    // TEST SERVERS
    '7125': { cancellationFee:'XXXXX',
        futureCredit:'XXXXX' },
    '7126': { cancellationFee:'XXXXX',
        futureCredit:'XXXXX' }
}

exports.cancelRez = async (config, page, cancelCase) => {
    // logger.info('config ' + JSON.stringify(config))
    logger.info('1655.cancellationFee ' + dynamicPageSettings[config.id].cancellationFee)    

    // logger.info(config.url + 'WebAgency/Reservation/OpenReservation/Details.aspx?id=' + cancelCase.reservationId)
    await page.goto(config.url + 'WebAgency/Reservation/OpenReservation/Details.aspx?id=' + cancelCase.reservationId, {waitUntil: 'load'});
    await page.waitForSelector('#btnCancelReservation')

    const [popup] = await Promise.all([
        new Promise(resolve => page.once('dialog', dialog => {
            dialog.accept()
            resolve()
        })),

        await page.click('#btnCancelReservation')
    ]);

    await page.waitForSelector('#lstBookingCancellationReason')
    await page.select('#lstBookingCancellationReason', '1005'); // Customer unable to take vacation (meh)

    await page.waitForSelector('#txtBookingCancellationReasonNotes')
    await page.focus('#txtBookingCancellationReasonNotes');
    await page.keyboard.type('Robot Cancellation');

    await page.waitForSelector('#btnCancelReservation')
    await page.click('#btnCancelReservation')

    // waits to see if we see success message
    await page.waitForSelector('#Form1 > div.panel > div.panelContent > table > tbody > tr > td > div:nth-child(1)')
    const statusTxt = await page.$('#Form1 > div.panel > div.panelContent > table > tbody > tr > td > div:nth-child(1)');
    const statusTxtMsg = await page.evaluate(statusTxt => statusTxt.textContent, statusTxt);
    logger.info('Success? ' + statusTxtMsg)

    if ( statusTxtMsg != 'Success!' ) {
        const statusErrElem = await page.$('#Form1 > div.panel > div.panelContent > table > tbody > tr > td > div:nth-child(2)')
        // logger.info('wtf failure? statusErrElem: ' + JSON.stringify(statusErrElem))
        cancelCase.cancelErr = await page.evaluate(statusErrElem => statusErrElem.textContent, statusErrElem);
        logger.info('wtf failure? cancelCase.cancelErr: ' + JSON.stringify(cancelCase.cancelErr))
        cancelCase.cancelErr = cancelCase.cancelErr.replace(/(\r\n|\n|\r)/gm," ");
        return false
    }

    // logger.info('CANCELREZ? ' + cancelCase.channel + ';' + cancelCase.Type)
    if ( cancelCase.channel == 'TK' && (cancelCase.Type == '10% future credit' || cancelCase.Type == '100% future credit') ) {
        // we are charging guest 10% cancellation fee - guest will be notified they have a future credit and
        // credit case will be created for 10% of rez value
        
        await page.goto(config.url + `WebAgency/Reservation/OpenReservation/Folios.aspx?id=${cancelCase.reservationId}`)
        await doFutureStayCreditCharge (config, page, cancelCase)
    }

    else if ( cancelCase.channel == 'TK' && cancelCase.Type == 'No refund or credit' ) {
        await page.goto(config.url + `WebAgency/Reservation/OpenReservation/Folios.aspx?id=${cancelCase.reservationId}`)
        await doNoCreditOrRefund (config, page, cancelCase)
    }

    else if ( cancelCase.Type == '100% Guest refund' ) {
        await page.goto(config.url + `WebAgency/Reservation/OpenReservation/Folios.aspx?id=${cancelCase.reservationId}`)
        await doFullRefundCreditCard (page, cancelCase)
    }

    return true
}

async function doNoCreditOrRefund (config, page, cancelCase) {
    logger.info('doNoCreditOrRefund 2: ' + cancelCase.Id)
    await page.waitForSelector('#folioToolbarTable > tbody > tr > td:nth-child(8)')
    const [popup] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        page.click('#folioToolbarTable > tbody > tr > td:nth-child(8)')
    ]);

    await retry(
        async () => {
            logger.info('cancelRez 3: ' + cancelCase.Id)
            await popup.waitForSelector('#lstDocumentTemplate'),
            await popup.select('#lstDocumentTemplate', dynamicPageSettings[config.id].cancellationFee), // 39847 - Cancellation fee; 96217 - Future credit
            await popup.waitForSelector('#txtAmount')
        },
        3) // retry this 3 times

    logger.info('cancelCase.payments: ' + cancelCase.payments)
    await popup.evaluate((chargeAmt) => {
        document.querySelector('#txtAmount').value = chargeAmt.toString()
        return true
    }, cancelCase.payments);

    logger.info('doNoCreditOrRefund 5: ' + cancelCase.Id)
    await popup.waitForSelector('#btnSubmit')
    await popup.click('#btnSubmit', {waitUntil: 'networkidle0'})
    await sleep(10000)

    logger.info('doNoCreditOrRefund 6: ' + cancelCase.Id)
    return
}

async function doFutureStayCreditCharge (config, page, cancelCase) {
    logger.info('doFutureStayCreditCharge 2')
    
    await page.waitForSelector('#folioToolbarTable > tbody > tr > td:nth-child(8)')
    const [popup] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        page.click('#folioToolbarTable > tbody > tr > td:nth-child(8)')
    ]);

    await retry(
        async () => {
            logger.info('cancelRez 3')
            await popup.waitForSelector('#lstDocumentTemplate'),
            await popup.select('#lstDocumentTemplate', dynamicPageSettings[config.id].futureCredit), // 39847 - Cancellation fee; 96217 - Future credit
            await popup.waitForSelector('#txtAmount')
        },
        3) // retry this 3 times

    // logger.info('cancelRez 4')
    await popup.evaluate((chargeAmt) => {
        document.querySelector('#txtAmount').value = chargeAmt.toString()
        return true
    }, cancelCase.payments);

    // logger.info('cancelRez 5')
    await popup.waitForSelector('#btnSubmit')
    await popup.click('#btnSubmit', {waitUntil: 'networkidle0'})
    await sleep(10000)

    // logger.info('cancelRez 6')
    return
}

async function doFullRefundCreditCard (page, cancelCase) {
    logger.info('doFullRefundCreditCard')

    await page.waitForSelector('#folioToolbarTable > tbody > tr > td:nth-child(7)')
    const [popup] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        page.click('#folioToolbarTable > tbody > tr > td:nth-child(7)')
    ]);

    await popup.waitForSelector('#lstPaymentType')
    await popup.select('#lstPaymentType', '2')
    
    await popup.waitForSelector('#lstCreditCard')
    let options = await getSelectOptions(popup, 'select#lstCreditCard > option' );
    await popup.select('select#lstCreditCard', options.values[0]);

    await sleep(3000)
    await popup.waitForSelector('#rptRefundables_ctl01_chkSelect')
    await popup.click('#rptRefundables_ctl01_chkSelect')

    await sleep(3000)
    await popup.click('#frmFolioRefund > table > tbody > tr:nth-child(1) > td > div > div:nth-child(5) > div > table > tbody > tr')
    await popup.click('#frmFolioRefund > table > tbody > tr:nth-child(1) > td > div > div:nth-child(5) > div > table > tbody > tr')

    await sleep(3000)
    popup.click('#btnGenerateRefund')

    await sleep(10000)
    logger.info('end doFullRefundCreditCard')

    return
}