const env = require('@turnkeyvr/env');
const logger = require('@turnkeyvr/logger');
// const username = env.config.get('robotify.escapia.username');
// const password = env.config.get('robotify.escapia.password');
// const securityQuestionAnswer = env.config.get('robotify.escapia.security-question-answer');
const { sleep, resultsEnum } = require('../util/misc');
const { EscapiaConfigManager, CONFIG_TYPES } = require('@turnkeyvr/escapia');
let _currentConfig = null

exports.setAndGetConfigEscapia = (instanceId) => {
    _currentConfig = EscapiaConfigManager.get({id: instanceId, type: CONFIG_TYPES.ROBOT})
    return _currentConfig
}

exports.authenticateWithEscapia = async (page) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        logger.info('_currentConfig: ' + JSON.stringify(_currentConfig.url))
        
        await page.goto(_currentConfig.url + 'WebAgency/Account/LogOn', {waitUntil: 'load', timeout: 60 * 1000 * 2});

        // fill in username
        await page.waitForSelector('#UserName', {visible: true});
        await page.focus('#UserName');
        await page.keyboard.type(_currentConfig.user);

        // fill in pw
        await page.waitForSelector('#Password', {visible: true});
        await page.focus('#Password');
        await page.keyboard.type(_currentConfig.password);

        // click login button
        await page.keyboard.press('Enter');
        await page.waitForNavigation({waitUntil: 'load'});

        if (page.url().indexOf('SecurityQuestionChallenge') > -1) {
            // fill in security question
            await page.waitForSelector('#Answer', {visible: true});
            await page.focus('#Answer');
            await page.keyboard.type(_currentConfig.securityQuestionAnswer);

            await page.keyboard.press('Enter');
            await page.waitForNavigation({waitUntil: 'load'});
        }

        let checkUrl = 'https://app.escapia.com/p/dashboard'
        if (page.url().toLowerCase() !== checkUrl) {
            logger.error('authenticate with escapia: login failed');
            return null;
        }

        return page;
    }
    catch (e) {
        logger.error('authenticateWithEscapia failed: ', e);
        return null;
    }
};

exports.logoutOfEscapia = async (page) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    await page.goto(_currentConfig.url + 'webagency/Logout.aspx', {waitUntil: 'load'});
    return page;
};

// testing issue in airbnb rate reconciliation
exports.logoutOfEscapia2 = async (page) => {
    if ( !_currentConfig ) {
        throw 'Must .setAndGetConfig() before calling this method!'
    }

    await page.goto(_currentConfig.url + 'webagency/Logout.aspx', { waitUntil: 'load', timeout: 30 * 1000 });
    return page;
};

// {waitUntil: 'load', timeout: 0}
exports.navigateToReservation = async (page, escapiaUnitReservationId) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    await page.goto(_currentConfig.url + 'webagency/Reservation/OpenReservation/Details.aspx?id=' + escapiaUnitReservationId + '&ReturnUrl=%2fWebAgency%2fReservation%2fGrid.aspx%3fRunFilter%3dtrue', {waitUntil: 'load'});
    return page;
};

exports.navigateToBooking = async (page, escapiaUnitReservationId) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    await page.goto(_currentConfig.url + 'webagency/Reservation/OpenReservation/Bookings.aspx?id=' + escapiaUnitReservationId + '&ReturnUrl=%2fWebAgency%2fReservation%2fGrid.aspx%3fRunFilter%3dtrue', {waitUntil: 'load'});
    return page;
};

exports.openBooking = async (page, escapiaUnitReservationId) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    page = await this.navigateToBooking(page, escapiaUnitReservationId); // has to navigate to the booking before it can be opened
    let content = await page.content();
    let bookingId = content.match('bookingid=(.*?)&amp;ReturnURL')[1]; // this booking id is not easily available in sfdc so using this regex to extract from the page
    await page.goto(_currentConfig.url + '/webagency/Reservation/EditBooking.aspx?bookingid=' + bookingId + '&amp;ReturnURL=%2fwebagency%2fReservation%2fOpenReservation%2fBookings.aspx%3fid%3d9916183%26ReturnUrl%3d%252fWebAgency%252fReservation%252fGrid.aspx%253fRunFilter%253dtrue', {waitUntil: 'load', timeout: 0});
    return page;
};

exports.navigateToFolio = async (page, escapiaUnitReservationId) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    await page.goto(_currentConfig.url + 'webagency/Reservation/OpenReservation/Folios.aspx?id=' + escapiaUnitReservationId + '&ReturnUrl=%2fWebAgency%2fReservation%2fGrid.aspx%3fRunFilter%3dtrue', {waitUntil: 'load'});
    return page;
};

exports.applyHostFeeCreditToFolio = async (page, escapiaUnitReservationId, listingHostFee) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        page = await this.navigateToFolio(page, escapiaUnitReservationId);
        let folioId = await page.evaluate(() => _folioID);   // window JS variable
        let officeId = await page.evaluate(() => _officeID); // window JS variable
        if (!folioId || !officeId) {
            logger.error('unable to retrieve either/or folioId + officeId from folio page');
            return null;
        }
        await page.goto(_currentConfig.url + 'webagency/Accounting/AccountsReceivable/FolioCredit.aspx?FolioID=' + folioId + '&OfficeID=' + officeId + '&isPopupDialog=0', {waitUntil: 'load'});
    
        // select document template
        await page.select('#lstDocumentTemplate', '76339');
    
        // fill in listingHostFee
        await page.waitForSelector('#txtAmount'); // selecting the document template above causes reload of input; wait for load
        await page.focus('#txtAmount');
        await page.keyboard.type(listingHostFee.toString());

        // click on Post Credit Button
        await page.click('#btnSubmit', {waitUntil: 'load'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error(e);
        return resultsEnum.FAILURE;
    }
};

// not documented in https://turnkeyvr.screenstepslive.com/s/internal/m/77430/l/871336-finalize-airbnb-reservation
exports.applyTaxRateHitToFolio = async (page, escapiaUnitReservationId, taxRateHitAmount) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        // TK Escapia: "TKVR hit - tax rate changes [TKVR hit - tax rate changes]"
        // Mville Escapia: "ACE Hit - tax rate changes [ACE Hit - tax rate changes]"
        let documentTemplateId = _currentConfig.url === 'https://vstays.escapia.com/' ? '101042' : '68596';

        page = await this.navigateToFolio(page, escapiaUnitReservationId);
        let folioId = await page.evaluate(() => _folioID);   // window JS variable
        let officeId = await page.evaluate(() => _officeID); // window JS variable
        if (!folioId || !officeId) {
            logger.error('unable to retrieve either/or folioId + officeId from folio page');
            return null;
        }
        await page.goto(_currentConfig.url + 'webagency/Accounting/AccountsReceivable/FolioCredit.aspx?FolioID=' + folioId + '&OfficeID=' + officeId + '&isPopupDialog=0', {waitUntil: 'load'});
    
        // select document template
        await page.select('#lstDocumentTemplate', documentTemplateId);
    
        // fill in listingHostFee
        await page.waitForSelector('#txtAmount'); // selecting the document template above causes reload of input; wait for load
        await page.focus('#txtAmount');
        await page.keyboard.type(taxRateHitAmount.toString());

        // click on Post Credit Button
        await page.click('#btnSubmit', {waitUntil: 'load'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error(e);
        return resultsEnum.FAILURE;
    }
};

exports.applyCommissionChargeToFolio = async (page, escapiaUnitReservationId, commissionFolioCharge) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        page = await this.navigateToFolio(page, escapiaUnitReservationId);
        let folioId = await page.evaluate(() => _folioID);   // window JS variable
        let officeId = await page.evaluate(() => _officeID); // window JS variable
        if (!folioId || !officeId) {
            logger.error('unable to retrieve either/or folioId + officeId from folio page');
            return null;
        }
        await page.goto(_currentConfig.url + 'webagency/Accounting/AccountsReceivable/NewFolioCharge.aspx?FolioID=' + folioId + '&OfficeID=' + officeId + '&isPopupDialog=0', {waitUntil: 'load'});
    
        // select document template
        await page.select('#lstDocumentTemplate', '100863');
    
        // fill in commissionFolioCharge
        await page.waitForSelector('#txtAmount'); // selecting the document template above causes reload of input; wait for load
        await page.focus('#txtAmount');
        await page.keyboard.type(commissionFolioCharge.toString());

        // click on Post Credit Button
        await page.click('#btnSubmit', {waitUntil: 'load'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error(e);
        return resultsEnum.FAILURE;
    }
};

exports.handleAirbnbEscapiaPercentComparison = async (browser, page, escapiaUnitReservationId, quotePercent, airbnbAmountPlusListingHostFee) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        page = await this.openBooking(page, escapiaUnitReservationId);

        let proxyKey = await page.evaluate(() => g_proxyKey); // window JS variable
        if (!proxyKey) {
            logger.error('Failed to retrieve proxyKey in escapiaUtil.handleAirbnbEscapiaPercentComparison');
            return resultsEnum.FAILURE;
        }

        logger.debug('quotePercent: ', quotePercent);

        // TODO: what is step 4 under the security deposits page??? "Adjust the rent to match the correct Airbnb payout."

        // TODO: confirm there can only be one security deposit line item
        // NOTE: if Escapia changes how the page is setup, there's really no way to detect a failure here...
        let depositRadioIndex = await page.$eval('#depositsContent', (depositsDiv) => {
            let depositRadioIndex = -1;
            let chargeTdArray = depositsDiv.getElementsByClassName('charge');
            for (let i = 0; i < chargeTdArray.length; i++) {
                let td = chargeTdArray[i];
                if (!td.textContent || !td.textContent.toLowerCase().includes('security deposit')) {
                    continue;
                }
                let inputArray = td.parentElement.getElementsByTagName('input');
                if (inputArray.length === 0) {
                    return depositRadioIndex;
                }
                depositRadioIndex = inputArray[0].value;
                break;
            }
            return depositRadioIndex;
        });
        logger.debug("TCL: exports.handleAirbnbEscapiaPercentComparison -> depositRadioIndex", depositRadioIndex);


        // NOTE: as of 11-8-19, haven't been able to test this logic because security deposits are evidently rare;
        //       for now, going to let Smiles handle Cases with security deposits.
        // TODO: probably need to open this in a new tab
        if (depositRadioIndex > -1) {
            return resultsEnum.FAILURE;
            // await page.goto('https://turnkeyvr.escapia.com/Webagency/Reservation/OverrideDeposit.aspx?ProxyKey=' + proxyKey + '&Index=' + depositRadioIndex + '&isPopupDialog=0', {waitUntil: 'load'});
            // // TODO: click delete button
            // // TODO: confirm Id accurate
            // await page.click('#btnDelete', {waitUntil: 'load'});
            // await sleep(10000); // temp for testing
        }

        // >= 85% logic
        if (quotePercent >= 85) {
            // set window.g_chargeRadioIndex to -1
            let g_chargeRadioIndex = await page.evaluate(() => {
                window.g_chargeRadioIndex = -1;
                return window.g_chargeRadioIndex;
            });
            if (g_chargeRadioIndex !== -1) {
                logger.error('Failed to set g_chargeRadioIndex in handleAirbnbEscapiaPercentComparison');
                return resultsEnum.FAILURE;
            }

            // processing loop, may take several iterations
            let sameAdjustmentCount = 0;
            let mostRecentAdjustmentAmount;
            while (true) {
                // get booking total
                let bookingTotal = await page.$$eval('.dollarDesc', tdArray => {
                    for (let i = 0; i < tdArray.length; i++) {
                        let td = tdArray[i];
                        if (!td.textContent || td.textContent.toLowerCase().trim() !== 'booking total') {
                            continue;
                        }
                        let dollarAmtArray = td.parentElement.getElementsByClassName('dollarAmt');
                        if (dollarAmtArray.length !== 1) {
                            return null;
                        }
                        return dollarAmtArray[0].textContent;
                    }
                    return null;
                });
                if (!bookingTotal) {
                    logger.error('Failed to get booking total');
                    return resultsEnum.FAILURE;
                }
                // parse value
                bookingTotal = Number(bookingTotal.replace(/[^0-9.-]+/g,""));
                logger.debug('bookingTotal: ', bookingTotal);

                // get rent charge
                let rentCharge = await page.$$eval('.charge', tdArray => {
                    for (let i = 0; i < tdArray.length; i++) {
                        let td = tdArray[i];
                        if (!td.textContent || td.textContent.toLowerCase().trim() !== 'rent charge') {
                            continue;
                        }
                        let dollarAmtArray = td.parentElement.getElementsByClassName('dollarAmt');
                        if (dollarAmtArray.length !== 2) {
                            return null;
                        }
                        return dollarAmtArray[0].textContent;
                    }
                    return null;
                });
                if (!rentCharge) {
                    logger.error('Failed to get booking total');
                    return resultsEnum.FAILURE;
                }
                // parse value
                rentCharge = Number(rentCharge.replace(/[^0-9.-]+/g,""));
                logger.debug('rentCharge: ', rentCharge);

                logger.debug('airbnbAmountPlusListingHostFee: ', airbnbAmountPlusListingHostFee);

                // do override calculations
                let overrideAmount;
                let adjustmentAmount;
                if (bookingTotal > airbnbAmountPlusListingHostFee) {
                    adjustmentAmount = bookingTotal - airbnbAmountPlusListingHostFee;
                    overrideAmount = rentCharge - adjustmentAmount;
                    
                }
                else if (bookingTotal < airbnbAmountPlusListingHostFee) {
                    adjustmentAmount = airbnbAmountPlusListingHostFee - bookingTotal;
                    overrideAmount = rentCharge + adjustmentAmount;
                }
                else {
                    // no action is needed, early exit
                    return resultsEnum.SUCCESS;
                }
                overrideAmount = Number(Math.round(overrideAmount + 'e2') + 'e-2'); // https://www.jacklmoore.com/notes/rounding-in-javascript/
                logger.debug('overrideAmount2: ', overrideAmount);

                adjustmentAmount = Number(Math.round(adjustmentAmount + 'e2') + 'e-2');
                if (adjustmentAmount < 0) {
                    adjustmentAmount *= -1; // always use the positive difference
                }


                // reset values if different, otherwise increase the sameAdjustmentCount
                if (mostRecentAdjustmentAmount !== adjustmentAmount) {
                    mostRecentAdjustmentAmount = adjustmentAmount;
                    sameAdjustmentCount = 0;
                }
                else {
                    sameAdjustmentCount += 1;
                }

                logger.debug('adjustmentAmount: ', adjustmentAmount);
                logger.debug('sameAdjustmentCount ', sameAdjustmentCount);
                logger.debug('mostRecentAdjustmentAmount: ', mostRecentAdjustmentAmount);

                // sometimes, even adjusting the rentCharge by 0.01 (either positively or negatively) results in the bookingTotal always being 0.01 (or more) off from airbnbAmountPlusListingHostFee
                // if sameAdjustmentCount hits 3, that means the bookingTotal cannot be exactly the airbnb amount + listing host fee
                // in that case, we need to wait until the bookingTotal is airbnbAmountPlusListingHostFee + 0.01, then add an additional credit to the folio for the mostRecentAdjustmentAmount
                // NOTE: this is not documented in https://turnkeyvr.screenstepslive.com/s/internal/m/77430/l/871336-finalize-airbnb-reservation
                if (sameAdjustmentCount >= 3 && bookingTotal > airbnbAmountPlusListingHostFee) {
                    // if mostRecentAdjustmentAmount is great than a few cents, something is probaby wrong
                    if (mostRecentAdjustmentAmount > 0.05) {
                        logger.error('mostRecentAdjustmentAmount is greater than 5 cents');
                        return resultsEnum.FAILURE;
                    }

                    return await this.applyTaxRateHitToFolio(page, escapiaUnitReservationId, mostRecentAdjustmentAmount);
                }

                await page.click('#btnOverride', {waitUntil: 'load'});
                await page.waitForSelector('#overlayIframe', {visible: true, timeout: 0});

                await sleep(20000); // todo: right now this is required; bad hack, needs to go

                let frames = page.mainFrame().childFrames();
                let myFrame = frames.find(f => f.url().toLowerCase().indexOf('overriderent.aspx') > -1);
                if (!myFrame) {
                    logger.error('Failed to get handle to override rent modal');
                    return resultsEnum.FAILURE;
                }

                logger.debug('iframe loaded');

                await myFrame.waitForSelector('#txtAmount', {visible: true, timeout: 0});
                await myFrame.type('#txtAmount', '' + overrideAmount);

                // click save button on both iframe and the main page
                await myFrame.click('#btnSave', {waitUntil: 'networkidle0'});

                await sleep(10000); // todo: right now this is required; bad hack, needs to go

                await page.click('#btnSave', {waitUntil: 'networkidle0'});

                await sleep(6000); // todo: right now this is required; bad hack, needs to go
            }
        }

        // < 85%; apply 15% discount
        logger.debug('about to apply 15% discount...');

        page = await this.openBooking(page, escapiaUnitReservationId); // open booking

        // per Conner, first reset the rate
        await page.click('#btnRemoveRateOverride', {waitUntil: 'networkidle0'});
        await sleep(3000); // todo: right now this is required; bad hack, needs to go

        await page.focus('#txtDiscountAmount');
        await page.keyboard.type('15.00');
        await page.click('#btnApplyDiscount', {waitUntil: 'networkidle0'});

        await sleep(3000); // todo: right now this is required; bad hack, needs to go

        await page.click('#btnSave', {waitUntil: 'networkidle0'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('handleAirbnbEscapiaPercentComparison failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.handleTaxEdits = async (browser, page, escapiaUnitReservationId, taxEdits) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    try {
        // early exit if tax edits aren't required
        if (!taxEdits) {
            return resultsEnum.SUCCESS;
        }

        logger.debug('taxEdits: ', taxEdits);

        page = await this.openBooking(page, escapiaUnitReservationId);

        // Tax Exempt logic
        if (taxEdits.action === 'Tax Exempt') {
            await page.select('#lstTaxableType', '2');

            await sleep(3000); // temp logic for testing

            await page.click('#btnSave', {waitUntil: 'load'});

            // early exit if tax exempt
            return resultsEnum.SUCCESS;
        }

        // Partial Taxes logic
        let proxyKey = await page.evaluate(() => g_proxyKey); // window JS variable
        if (!proxyKey) {
            logger.error('Failed to retrieve proxy key for handleTaxEdits');
            return resultsEnum.FAILURE;
        }

        // bugfix: need to open tax edits in new tab, then save the booking
        let taxEditPage = await browser.newPage();
        await taxEditPage.goto(_currentConfig.url + 'webagency/Reservation/ViewTaxes.aspx?ProxyKey=' + proxyKey + '&isPopupDialog=0', {waitUntil: 'load'});

        let taxesToBeAdjustedSubcategories = taxEdits.taxesToBeAdjusted.map(tax => tax.subcategory);

        // todo: once sleep removed doesn't need to be async
        let adjustedTaxesResults = await taxEditPage.evaluate(async (taxesToBeAdjustedSubcategories, resultsEnum) => {
            console.log("TCL: exports.handleTaxEdits -> taxesToBeAdjustedSubcategories", taxesToBeAdjustedSubcategories)
            const sleep = ms => new Promise(resolve => setTimeout(resolve, ms));

            try {
                let buttonNodeList = document.querySelectorAll('button');
                let buttonArray = Array.from(buttonNodeList);
                let taxesOverridden = false;

                // process each override button
                buttonArray.forEach(button => {
                    if (button.textContent !== 'Override') {
                        return;
                    }
    
                    window.doEditTax(button); // Escapia JS method
                    let taxSubcategory = window._row.cells[2].textContent.trim();
                    if (!taxesToBeAdjustedSubcategories.includes(taxSubcategory)) {
                        window.doValueSet();
                        return;
                    }
    
                    window._row.editAmountEl.value = 0.01;
                    window.doValueSet(); // Escapia JS method

                    taxesOverridden = true; // TODO: consider making this check more specific
                });

                await sleep(10000); // temp logic for testing

                if (taxesOverridden) {
                    return resultsEnum.SUCCESS;
                }
                return resultsEnum.REVIEW_REQUIRED; // returning REVIEW_REQUIRED because taxes were not overridden
            }
            catch (e) {
                return resultsEnum.FAILURE;
            }
        }, taxesToBeAdjustedSubcategories, resultsEnum);

        if (adjustedTaxesResults === resultsEnum.FAILURE) {
            return resultsEnum.FAILURE;
        }

        // click save button on both tax edit page + booking page
        await taxEditPage.click('#btnSave', {waitUntil: 'networkidle0'});
        await taxEditPage.close();
        await page.click('#btnSave', {waitUntil: 'networkidle0'});

        return adjustedTaxesResults;
    }
    catch (e) {
        logger.error('handleTaxEdits failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.checkForPendingReconciliationReservationNote = async (page, escapiaUnitReservationId) => {
    try {
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let pendingReconciliationNotePresent = await page.evaluate(() => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            if (resNotesInput.value.indexOf('[Pending Reconciliation]') > -1) {
                return true;
            }
            return false;
        });

        if (!pendingReconciliationNotePresent) {
            return resultsEnum.FAILURE;
        }

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('checkForPendingReconciliationReservationNote failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.removePendingReconciliationReservationNote = async (page, escapiaUnitReservationId) => {
    try {
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let pendingReconciliationNoteRemoved = await page.evaluate(() => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            if (resNotesInput.value.indexOf('[Pending Reconciliation]') > -1) {
                resNotesInput.value = resNotesInput.value.replace('[Pending Reconciliation]', '');
                return true;
            }
            return false;
        });

        if (!pendingReconciliationNoteRemoved) {
            return resultsEnum.FAILURE;
        }

        await page.click('#btnSave', {waitUntil: 'load'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('removePendingReconciliationReservationNote failed: ', e);
        return resultsEnum.FAILURE;
    }
};

// Sometimes Escapia thinks international phone numbers in the reservation note are credit card numbers, 
// and won't allow the reservation to be saved until it's removed. Since we don't need the phone number
// in the reservation note at all at this point, just remove it every time.
exports.removeGuestPhoneNumberFromReservationNote = async (page, escapiaUnitReservationId, guestPhoneNumber) => {
    try {
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let guestPhoneNumberNoteRemoved = await page.evaluate((guestPhoneNumber) => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            if (resNotesInput.value.indexOf('Guest Phone Number:') > -1) {
                resNotesInput.value = resNotesInput.value.replace('Guest Phone Number: ' + guestPhoneNumber, '');
                return true;
            }
            return false;
        }, guestPhoneNumber);

        if (!guestPhoneNumberNoteRemoved) {
            return resultsEnum.FAILURE;
        }

        await page.click('#btnSave', {waitUntil: 'load'});

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('removeGuestPhoneNumberFromReservationNote failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.checkForProcessedNoteInReservation = async (page, escapiaUnitReservationId) => {
    try {
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let processedNoteAddedComplete = await page.evaluate(() => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            return resNotesInput.value.indexOf('[RECONCILIATION COMPLETE]') > -1;
        });

        if (!processedNoteAddedComplete) {
            return resultsEnum.FAILURE;
        }

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('addProcessedNoteToReservation failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.addProcessedNoteToReservation = async (page, escapiaUnitReservationId) => {
    try {
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let processedNoteAdded = await page.evaluate(() => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            resNotesInput.value = `[RECONCILIATION COMPLETE]\n\n${resNotesInput.value}`;
            return true;
        });

        if (!processedNoteAdded) {
            return resultsEnum.FAILURE;
        }

        await page.click('#btnSave', {waitUntil: 'load'});

        // then need to confirm it was successfully added; if not, return an error
        page = await this.navigateToReservation(page, escapiaUnitReservationId);

        let processedNoteAddedComplete = await page.evaluate(() => {
            let resNotesInput = document.getElementById('txtReservationNotes');
            if (!resNotesInput) {
                return false;
            }
            return resNotesInput.value.indexOf('[RECONCILIATION COMPLETE]') > -1;
        });

        if (!processedNoteAddedComplete) {
            return resultsEnum.FAILURE;
        }

        return resultsEnum.SUCCESS;
    }
    catch (e) {
        logger.error('addProcessedNoteToReservation failed: ', e);
        return resultsEnum.FAILURE;
    }
};

exports.getReservationNumberFromReservation = async (page, escapiaUnitReservationId) => {
    page = await this.navigateToReservation(page, escapiaUnitReservationId);

    let reservationNumber = await page.$$eval('.lineAmt', (elemArray) => {
        elemArray = elemArray.filter(elem => elem.textContent && elem.textContent.startsWith('RES-'));
        if (elemArray.length > 0) {
            return elemArray[0].textContent;
        }
        return null;
    });

    return reservationNumber;
};

exports.getIsCancelledFromReservation = async (page, escapiaUnitReservationId) => {
    page = await this.navigateToReservation(page, escapiaUnitReservationId);

    let isCancelled = await page.$$eval('.lineAmt', (elemArray) => {
        elemArray = elemArray.filter(elem => elem.textContent && elem.textContent.trim() === 'Canceled');
        if (elemArray.length > 0) {
            return true;
        }
        return false;
    });

    return isCancelled;
};

exports.searchForUserByEmail = async (page, email, role) => {
    if ( !_currentConfig )
        throw 'Must .setAndGetConfig() before calling this method!'

    // open contact search page
    let url = role ? _currentConfig.url + 'WebAgency/FindContacts/Find?Role=' + role : _currentConfig.url + 'WebAgency/FindContacts/Find?Role=None';
    await page.goto(url, {waitUntil: 'networkidle0'});

    // input email
    await page.focus('#txtEmail');
    await page.keyboard.type(email);

    // click search button
    await page.click('#butSearch', {waitUntil: 'networkidle0'});

    await sleep(5000); // terrible, bullshit hack; need to replace this

    // click row
    let rowCount = await page.evaluate(() => {
        let elements = document.getElementsByClassName('clickable');
        if (!elements.length || elements.length !== 1) {
            return elements.length;
        }
        elements[0].children[1].click();
        return 1;
    });

    // error handling for no results or multiple results found
    if (rowCount !== 1) {
        return resultsEnum.FAILURE;
    }

    return page;
}