const { config } = require('@turnkeyvr/env');
const MongoClient = require('mongodb').MongoClient;
const logger = require('@turnkeyvr/logger');
const ObjectID = require('mongodb').ObjectID

exports.find = async (collName, queryObj, options) => {
    let conn = await connect()
    const db = conn.db()
    const collection = db.collection(collName);

    if ( typeof options == 'undefined' )
        options = {}

    let result = await collection.find(queryObj, options).toArray()
    conn.close()

    return result
}

exports.insertMany = async (collName, insertArr, options) => {
    let conn = await connect()
    const db = conn.db()
    const collection = db.collection(collName);

    if ( typeof options == 'undefined' )
        options = {}

    let result = await collection.insertMany(insertArr, options)
    conn.close()

    return result
}

exports.updateOneById = async (collName, id, updateObj, options) => {
    let conn = await connect()
    const db = conn.db()
    const collection = db.collection(collName);

    if ( typeof options == 'undefined' )
        options = {}

    let result = await collection.updateOne({_id: new ObjectID(id)}, {$set: updateObj}, options)
    conn.close()

    return result
}

exports.updateManyById = async (collName, idArray, updateObj, options) => {
    let conn = await connect()
    const db = conn.db()
    const collection = db.collection(collName);

    if ( typeof options == 'undefined' )
        options = {}

    idArray = idArray.map(id => new ObjectID(id));

    let results = await collection.updateMany({_id: {$in: idArray}}, {$set: updateObj}, options)
    conn.close()

    return results
}

exports.deleteOneById = async (collName, id, options) => {
    let conn = await connect()
    const db = conn.db()
    const collection = db.collection(collName);

    if ( typeof options == 'undefined' )
        options = {}

    let result = await collection.deleteOne({_id: new ObjectID(id)}, options)
    conn.close()

    return result
}

async function connect() {
    let mongoData = config.get("mongodb");

    let conn = await MongoClient.connect(`mongodb://${mongoData.url}/${mongoData.dbName}?retryWrites=false`, {
        useUnifiedTopology: true,
        auth: {user:mongoData.options.auth.user, password:mongoData.options.auth.password },
        ssl: true
    })

    return conn
}
