const logger = require('@turnkeyvr/logger');
const chromium = require('chrome-aws-lambda');
const { retry } = require('./retry');
const { sleep } = require('../util/misc');
const IS_PROD = (process.env.NODE_ENV.toLowerCase() === 'prod' || process.env.NODE_ENV.toLowerCase() === 'production');

exports.getBrowser = async () => {
    try {
        let browser = await chromium.puppeteer.launch({
            args: chromium.args,
            defaultViewport: chromium.defaultViewport,
            executablePath: await chromium.executablePath,
            headless: (IS_PROD ? true : chromium.headless)
        });

        return browser;
    }
    catch (e) {
        logger.error('failed to get browser instance: ', e);
        return null;
    }
};

// TODO: remove this function when possible, shouldn't be necessary; use getSelectValue instead
exports.getSelectedValueFromSelectHandle = nodes => {
    for (let i = 0; i < nodes.length; i++) {
        if (nodes[i].selected) {
            return nodes[i].textContent.trim();
        }
    }
};

// todo: should I .dispose() of ElementHandles?
exports.getSelectValue = async (page, selector) => {
    await retry(
        () => Promise.all([
            page.waitForSelector(selector, {visible: true})
        ]), 5
    );
    let selectHandle = await page.$(selector);
    let selectValue = await selectHandle.$$eval('option', nodes => {
        for (let i = 0; i < nodes.length; i++) {
            if (nodes[i].selected) {
                return nodes[i].textContent.trim();
            }
        }
    });
    return selectValue;
};

exports.getCheckboxValue = async (page, selector) => {
    await retry(
        () => Promise.all([
            page.waitForSelector(selector, {visible: true})
        ]), 5
    );
    let checkboxHandle = await page.$(selector);
    let checkboxValueHandle = await checkboxHandle.getProperty('checked');
    let checkboxValue = await checkboxValueHandle.jsonValue();
    return checkboxValue;
};

exports.getInputValue = async (page, selector) => {
    await retry(
        () => Promise.all([
            page.waitForSelector(selector, {visible: true})
        ]), 5
    );
    let inputHandle = await page.$(selector);
    let inputValueHandle = await inputHandle.getProperty('value');
    let inputValue = (await inputValueHandle.jsonValue()).trim();
    return inputValue;
};
exports.waitForReportSpinner = async (page) => {
    let waitSpinner = null
    do  {
        await sleep(1000)
        waitSpinner = await page.evaluate(async () => {
            return await new Promise(resolve => {
                resolve( document.querySelector('#ReportViewerControl_AsyncWait_Wait').offsetParent )
            })
        });
        // logger.info('waitSpinner? ' + JSON.stringify(waitSpinner))

    } while ( waitSpinner )
}
