'use strict';

const https = require('https');

/* usage
const url = require('url');
const { doRequest } = require('pathHere');

let response = await doRequest("https://httpbin.org/get")

// Get req w/ timeout and extra headers:
let response = await doRequest(Object.assign({}, url.parse("https://httpbin.org/get"), {timeout: 2000, headers: {'X-MyHeader': 'MyValue'}}))

// POST data:
let response = doRequest(Object.assign({}, url.parse("https://httpbin.org/post"), { method: 'POST', headers: {'Content-Type': 'application/x-www-form-urlencoded'}}), "foo=bar")
 */

exports.doRequest = async ((urlOptions, data) => {
    return new Promise((resolve, reject) => {
        const req = https.request(urlOptions,
            (res) => {
                let body = '';
                res.on('data', (chunk) => (body += chunk.toString()));
                res.on('error', reject);
                res.on('end', () => {
                    if (res.statusCode >= 200 && res.statusCode <= 299) {
                        resolve({statusCode: res.statusCode, headers: res.headers, body: body});
                    } else {
                        reject('Request failed. status: ' + res.statusCode + ', body: ' + body);
                    }
                });
            });
        req.on('error', reject);
        req.write(data, 'binary');
        req.end();
    });
});

