const logger = require('@turnkeyvr/logger');
const { sleep } = require('../util/misc');

let dynamicPageSettings = {
    // TK
    '1655': { serviceOrderCategory:'3316', // maintenance
        serviceOrderSubCategory:'4960' // Miscellaneous Maintenance
    },
    // MARG
    '2685': { serviceOrderCategory:'XXXX', // maintenance
        serviceOrderSubCategory:'XXXX' // Miscellaneous Maintenance
    },
    // TEST SERVERS
    '7125': { serviceOrderCategory:'XXXX', // maintenance
        serviceOrderSubCategory:'XXXX' // Miscellaneous Maintenance
    },
    '7126': { serviceOrderCategory:'XXXX', // maintenance
        serviceOrderSubCategory:'XXXX' // Miscellaneous Maintenance
    }
}

exports.createServiceOrder = async (config, page, serviceOrderCase) => {
    if ( config.id == '2685' )
        throw '2685 is not configured to do service orders!'

    await page.goto('https://turnkeyvr.escapia.com/webagency/ServiceOrder/NewServiceOrder/Step1.aspx', {waitUntil: 'load'});
    await page.waitForSelector('#ctlServiceOrderWorksheet_txtDescription')

    let description = serviceOrderCase.Bill_Owner_For__c
    if ( description == 'Other' ) 
        description = description + ': ' + serviceOrderCase.If_other_please_explain__c

    await page.evaluate((description, caseNumber) => {
        document.querySelector('#ctlServiceOrderWorksheet_txtDescription').value = description
        document.querySelector('#ctlServiceOrderWorksheet_txtReportedBy').value = 'ROBOT'
        document.querySelector('#ctlServiceOrderWorksheet_txtServiceRequested').value = caseNumber
        document.querySelector('#ctlServiceOrderWorksheet_txtCompletedBy').value = 'ROBOT'

        return true
    }, description, serviceOrderCase.CaseNumber);

    const [popup] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        page.click('#imgFindSLEntity')
    ]);

    await popup.waitForSelector('#inpVName')
    await popup.focus('#inpVName');
    
    await popup.keyboard.type(serviceOrderCase.OpportunityCase__r.Name);
    try {
        await popup.waitForSelector('#tblEntities > tbody > tr:nth-child(2) > td.listData.clickable')
    }
    catch (ex) {
        throw '[NO_RENTAL_FOUND]'
    }
    popup.click('#tblEntities > tbody > tr:nth-child(2) > td.listData.clickable')

    await page.focus('#ctlServiceOrderWorksheet_dpCompletedDate_Top > input')
    await page.keyboard.type('t');

    await page.select('#ctlServiceOrderWorksheet_lstCategory', dynamicPageSettings[config.id].serviceOrderCategory); 

    page.click('#btnNext')
    await page.waitForSelector('#tblToolbar > tbody > tr > td:nth-child(3)')
    
    let panelButtonArr = await page.$$('#tblToolbar > tbody > tr > td:nth-child(3)');

    const [popup2] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        panelButtonArr[0].click()
    ]);

    await popup2.waitForSelector('#txtName')
    await popup2.evaluate((description) => {
        document.querySelector('#txtName').value = description
        return true
    }, description);

    await popup2.select('#lstSubcategory', dynamicPageSettings[config.id].serviceOrderSubCategory);
    
    await popup2.focus('#dpCompletedDate_Top > input')
    await popup2.keyboard.type('t');

    await sleep(1000)
    popup2.click('#btnSave')

    // wait for task row from popup2 to appear
    await page.waitForSelector('#scrollDiv > table > tbody > tr')
    let enteredItems = await page.$$('#scrollDiv > table > tbody > tr');
  
    panelButtonArr = await page.$$('#tblToolbar > tbody > tr > td:nth-child(3)');

    const [popup3] = await Promise.all([
        new Promise(resolve => page.once('popup', resolve)),
        panelButtonArr[1].click()
    ]);
        
    await popup3.waitForSelector('#lstChargeTemplate')
    await popup3.select('#lstChargeTemplate', '20481');

    await popup3.waitForSelector('#txtCost')
        
    await popup3.focus('#txtCost')
    await popup3.keyboard.type(serviceOrderCase.Amount_to_bill_HomeOwner_for_service__c.toString());
    await popup3.focus('#txtTotalCost')
    popup3.click('#btnSave')

    let waitForCost = 0
    while ( waitForCost < 5 ) {
        await sleep(1000)
        enteredItems = await page.$$('#scrollDiv > table > tbody > tr');
        if ( enteredItems.length == 2 ) {
            waitForCost = 5
        }
        waitForCost++
    }

    if ( enteredItems.length != 2 ) {
        logger.error('could not enter task and cost!')
        return false
    }

    await sleep(1000)
    await page.click('#btnNext')
    
    let terribleSelectorToOpenServiceLink = '#frmSummary > div.panel > div.panelContent > table > tbody > tr > td > div:nth-child(3) > a:nth-child(4)'
    await page.waitForSelector(terribleSelectorToOpenServiceLink)

    let linkToServiceOrderUrl = await page.evaluate((link) => {
        let linkToServiceOrder = document.querySelector(link)
        return linkToServiceOrder.getAttribute('href')
    }, terribleSelectorToOpenServiceLink);

    linkToServiceOrderUrl = 'https://turnkeyvr.escapia.com/webagency/ServiceOrder' + linkToServiceOrderUrl.replace('..', '')
    logger.info('linkToServiceOrderUrl: ' + linkToServiceOrderUrl)
    await page.goto(linkToServiceOrderUrl, {waitUntil: 'load'});
    
    await page.waitForSelector('#frmEditServiceOrder > div.panel > div > table.fixed > tbody > tr > td:nth-child(1) > table > tbody > tr:nth-child(2) > td.lineAmt')
    let serviceOrderNumber = await page.evaluate(() => {
        let serviceorderNumber = document.querySelector('#frmEditServiceOrder > div.panel > div > table.fixed > tbody > tr > td:nth-child(1) > table > tbody > tr:nth-child(2) > td.lineAmt')
        return serviceorderNumber.innerText.trim()
    });
    logger.info('created serviceOrderNumber: ' + serviceOrderNumber + ' for case ' + serviceOrderCase.Id)
    
    await page.waitForSelector('#ctlServiceOrderWorksheet_btnCloseServiceOrder')
    const [popup4] = await Promise.all([
        new Promise(resolve => page.once('dialog', dialog => {
            dialog.accept()
            resolve()
        })),

        await page.click('#ctlServiceOrderWorksheet_btnCloseServiceOrder')
    ]);
    
    return serviceOrderNumber
}