const sfGen = require('@turnkeyvr/salesforcegeneral');
const logger = require('@turnkeyvr/logger');
const util = require('util');
const { resultsEnum } = require('../util/misc');

const querySF = util.promisify(sfGen.query);
const updateSF = util.promisify(sfGen.update);
const chatterNew = util.promisify(sfGen.chatter.new);

const STACKED_TAX_TYPE = 'STACKED_AIRBNB_COLLECTED_TAX'
const INELGIBLE_TAX_TYPE = 'INELIGIBLE';

// temp changes for testing: isclosed = true, DESC, LIMIT, Id
const airbnbRateReconciliationCaseQuery = 'SELECT '
    + 'Id, '
    + 'Subject, '
    + 'Description, '
    + 'Contact.Email, '
    + 'Contact.MobilePhone, '
    + 'Contact.HomePhone, '
    + 'Contact.OtherPhone, '
    + 'InquiryReceived__c, '
    + 'OpportunityCase__c, '
    + 'OpportunityCase__r.Name, '
    + 'OpportunityCase__r.Escapia_Instance_Id__c, '
    + 'OpportunityCase__r.Is_Marketing_Only__c, '
    + 'OpportunityCase__r.Rental_Setup__c, '
    + 'OpportunityCase__r.Rental_Setup__r.air_bnb_Integration_listing_live__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.Id, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.refresh_token__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.airbnb_access_code__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.Id, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.refresh_token__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.airbnb_access_code__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Using_Pass_Through_Taxes__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Pass_Through_Tax_Category__c, '
    + 'OpportunityCase__r.Airbnb_Is_Holding_Taxes_For_Rental__c, '
    + 'OpportunityCase__r.Tax_District_New__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Taxes_To_Be_Adjusted__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Taxes__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Partial_Taxes__c '
    + 'FROM Case '
    + 'WHERE Status = \'New\' '
    //+ 'AND Id = \'5000H00001SKgLD\' '
    + 'AND Subject = \'Finalize Airbnb reservation in Escapia\' '
    + 'AND RecordType.Name = \'Convert Hold To Reservation\' '
    + 'AND OpportunityCase__r.Escapia_Instance_Id__c = \'1655\' '
    + 'ORDER BY InquiryReceived__c ASC LIMIT 25';

// TODO: remove unneeded fields
// temp changes for testing: isclosed = true, DESC, LIMIT, Id
const googleRateReconciliationCaseQuery = 'SELECT '
    + 'Id, '
    + 'Subject, '
    + 'Description, '
    + 'Contact.Email, '
    + 'Contact.MobilePhone, '
    + 'Contact.HomePhone, '
    + 'Contact.OtherPhone, '
    + 'InquiryReceived__c, '
    + 'OpportunityCase__c, '
    + 'OpportunityCase__r.Escapia_Instance_Id__c, '
    + 'OpportunityCase__r.Is_Marketing_Only__c, '
    + 'OpportunityCase__r.Rental_Setup__c, '
    + 'OpportunityCase__r.Rental_Setup__r.air_bnb_Integration_listing_live__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.Id, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.refresh_token__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Integration_Account__r.airbnb_access_code__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.Id, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.refresh_token__c, '
    + 'OpportunityCase__r.Rental_Setup__r.Airbnb_Account__r.airbnb_access_code__c, '
    + 'OpportunityCase__r.Airbnb_Is_Holding_Taxes_For_Rental__c, '
    + 'OpportunityCase__r.Tax_District_New__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Taxes_To_Be_Adjusted__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Taxes__c, '
    + 'OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Partial_Taxes__c '
    + 'FROM Case '
    + 'WHERE Status = \'New\' '
    //+ 'AND Id = \'5000H00001SN58k\' '
    + 'AND Subject = \'Finalize Website reservation in Escapia\' '
    + 'AND RecordType.Name = \'Convert Hold To Reservation\' '
    + 'AND OpportunityCase__r.Escapia_Instance_Id__c = \'1655\' '
    + 'ORDER BY InquiryReceived__c ASC LIMIT 1';

const funBackingMarketingContentQuery = 'SELECT '
    + 'Id, '
    + 'Sync_Friendly_Unit_Name__c, '
    + 'Friendly_Unit_Name__c, '
    + '( '
        + 'SELECT '
        + 'Id, '
        + 'Escapia_Internal_Id__c '
        + 'FROM Opportunities__r '
        + 'WHERE Escapia_Internal_Id__c != null '
        + 'LIMIT 1'
    + ') '
    + 'FROM Backing_Marketing_Content__c '
    + 'WHERE Sync_Friendly_Unit_Name__c = true '
    + 'AND Friendly_Unit_Name__c != null '
    //+ 'AND Id = \'a1n0H000004r4uG\' '
    + 'LIMIT 1';

exports.getFUNBackingMarketingContentRecords = async () => {
    try {
        let results = await querySF(funBackingMarketingContentQuery);
        return results.records;
    }
    catch (e) {
        logger.error('getFUNBackingMarketingContentRecords failed: ', e);
        return null;
    }
};

exports.getRateReconciliationCases = async (reconciliationType) => {
    let queryString = reconciliationType === 'airbnb' ? airbnbRateReconciliationCaseQuery : googleRateReconciliationCaseQuery;
    try {
        let results = await querySF(queryString);
        return results.records;
    }
    catch (e) {
        logger.error('getRateReconciliationCases failed: ', e);
        return null;
    }
};

exports.getEscapiaUnitReservationIdFromCaseDescription = (description) => {
    if (description.indexOf('Escapia Unit Reservation Id:') === -1) {
        return null;
    }
    const matches = description.match('Escapia Unit Reservation Id: 1655-(\\d+)');
    return matches ? matches[1] : null;
};

// TODO: use Case.Airbnb_Escapia_Percent_Comparison__c instead
exports.getEscapiaQuotePercentComparisonFromCaseDescription = (description) => {
    if (description.indexOf('% Comparison (You earn amount to Escapia Quote)') === -1) {
        return null;
    }
    const matches = description.match(/% Comparison \(You earn amount to Escapia Quote\)\r?\n(\d+)%/);
    return matches ? parseInt(matches[1], 10) : null;
};

exports.getAirbnbAmountFromCaseDescription = (description) => {
    if (description.indexOf('Air bnb amount:') === -1) {
        return null;
    }
    const matches = description.match(/Air bnb amount:(.*?)\r?\n/);
    return matches ? Number(parseFloat(matches[1]).toFixed(2)) : null; // TODO: confirm two decimal places is kewl
};

exports.getWebsiteAmountFromCaseDescription = (description) => {
    if (description.indexOf('Website amount:') === -1) {
        return null;
    }
    const matches = description.match(/Website amount:(.*?)\r?\n/);
    return matches ? Number(parseFloat(matches[1]).toFixed(2)) : null; // TODO: confirm two decimal places is kewl
};

exports.getListingHostFeeFromCaseDescription = (description) => {
    if (description.indexOf('Listing Host Fee:') === -1) {
        return null;
    }
    const matches = description.match(/Listing Host Fee: (.*?)\r?\n/);
    return matches ? Number(parseFloat(matches[1]).toFixed(2)) : null; // TODO: confirm two decimal places is kewl
};

exports.getAirbnbConfirmationCodeFromCaseDescription = (description) => {
    if (description.indexOf('Airbnb Confirmation Code:') === -1) {
        return null;
    }
    const matches = description.match(/Airbnb Confirmation Code: (.*?)\r?\n/);
    return matches ? matches[1] : null;
};

exports.getGuestPhoneNumberFromCaseDescription = (description) => {
    if (description.indexOf('Guest Phone Number:') === -1) {
        return null;
    }
    const matches = description.match(/Guest Phone Number: (.*?)\r?\n/);
    return matches ? matches[1] : null;
};

// copied logic from turnkey-www/airbnbApi/notifications.js
exports.determineRequiredTaxEdits = (caseRecord) => {
    let oldHoldingTaxes = caseRecord.OpportunityCase__r.Airbnb_Is_Holding_Taxes_For_Rental__c ||
    (caseRecord.OpportunityCase__r.Tax_District_New__r != null && caseRecord.OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Taxes__c);
    let newHoldingTaxes = false;
    let partailTaxes  = false;

    if(caseRecord.OpportunityCase__r.Rental_Setup__r.Airbnb_Using_Pass_Through_Taxes__c && caseRecord.OpportunityCase__r.Rental_Setup__r.Airbnb_Pass_Through_Tax_Category__c  
        && caseRecord.OpportunityCase__r.Rental_Setup__r.Airbnb_Pass_Through_Tax_Category__c != INELGIBLE_TAX_TYPE)
    {
        newHoldingTaxes = caseRecord.OpportunityCase__r.Rental_Setup__r.Airbnb_Pass_Through_Tax_Category__c != STACKED_TAX_TYPE ;
        partailTaxes = caseRecord.OpportunityCase__r.Rental_Setup__r.Airbnb_Pass_Through_Tax_Category__c == STACKED_TAX_TYPE;
    }
    else
    {
        partailTaxes = (caseRecord.OpportunityCase__r.Tax_District_New__r != null && caseRecord.OpportunityCase__r.Tax_District_New__r.Airbnb_Processes_Partial_Taxes__c);
    }


    // ***Please set 'Tax Options' to 'Tax Exempt' from the dropdown menu in the Booking***
    if (!newHoldingTaxes && oldHoldingTaxes) {
        return { action: 'Tax Exempt' };
    }

    // ***Partial Tax District please follow the instructions below to adjust the reservation***
    if (partailTaxes) {
        let taxesToBeAdjusted = [];
        if (caseRecord.OpportunityCase__r.Tax_District_New__r.Taxes_To_Be_Adjusted__c) {
            taxesToBeAdjusted = caseRecord.OpportunityCase__r.Tax_District_New__r.Taxes_To_Be_Adjusted__c.split(/\r?\n/).map((str) => {
                return { name: str, subcategory: str.match(/(\d+.\d+)/)[1] };
            });
        }
        
        return { action: 'Partial Taxes', taxesToBeAdjusted: taxesToBeAdjusted };
    }

    return null;
};

exports.updateCase = async (caseRecord) => {
    try {
        await updateSF('Case', caseRecord);
    }
    catch (e) {
        logger.error('updateCase failed: ', e);
        return resultsEnum.FAILURE;
    }
    return resultsEnum.SUCCESS;
};

exports.chatterNew = async (data) => {
    try {
        await chatterNew(data);
    }
    catch (e) {
        logger.error('chatterNew failed: ', e);
        return resultsEnum.FAILURE;
    }
    return resultsEnum.SUCCESS;
};